#include <iostream>
#include "tests/Util.H"
#include <opencv2/highgui.hpp>

//! Loads an image, converts to floating point [0-255], and returns
//! the color image and its grayscale version
std::pair<cv::Mat, cv::Mat> loadImage( std::string const & file )
{
  auto image = cv::imread( file );
  // convert to float, leave in 0 to 255 range
  if( image.type() != CV_32FC3 )
    image.convertTo( image, CV_32F );

  cv::Mat grayscale;

  cv::cvtColor( image, grayscale, CV_BGR2GRAY );

  return {image, grayscale};
}


int main( int argc, char ** argv )
{
  if( argc != 3 )
  {
    std::cerr << "First arg: ground truth image, Second arg: all directions border ownership image" << std::endl;
    return 1;
  }

  auto groundTruth = loadImage(std::string(argv[1])).first;
  auto boImage = loadImage(std::string(argv[2])).first;

  if( groundTruth.cols != boImage.cols && groundTruth.rows != boImage.rows )
  {
    std::cerr << "Images must be the same size" << std::endl;
    return 1;
  }

  auto polarityScores = bd::test::activePolarityScore( groundTruth, boImage, 1.0f );
  auto polarityString = string_sprintf( "%f, %f, %f, %f, %f, %f",    std::get<0>(polarityScores),
                                        std::get<1>(polarityScores), std::get<2>(polarityScores),
                                        std::get<3>(polarityScores), std::get<4>(polarityScores),
                                        std::get<5>(polarityScores) );
  std::cout << polarityString << std::endl;

  return 0;
}
