function [] = RobustThresholdingDemo()

%% SPECIFICATIONS
lag         = 2;       % lag for the smoothing
threshold   = 2;     % number of st.dev. away from the mean to signal
influence   = 0.5;     % when signal: how much influence for new data? (between 0 and 1)
                       % 1 is normal influence, 0.5 is half      
%% START DEMO
% DemoScreen(30,lag,threshold,influence);
ConflictScreen(100);

end

function [fire, maxT, fastT, actT, decayT] = conflictThreshold(y, prevMaxT, prevFastT, prevActT, prevDecayT)

maxSmoothing = 0.25;
fastSmoothing = 0.5;
actSmoothing = 0.25;
decSmoothing = 0.25;
decayRate = 0.9;

decayT = prevDecayT;

if y > prevActT
    maxT = maxSmoothing * y + (1-maxSmoothing)*prevMaxT;   
    fastT = fastSmoothing * y + (1-fastSmoothing)*prevFastT;
else
    maxT = prevMaxT;    
    fastT = fastSmoothing * prevActT + (1-fastSmoothing)*prevFastT;
end

if y <= prevMaxT && y > prevDecayT
    actT = actSmoothing * y + (1-actSmoothing)*prevActT;
else
    actT = prevActT;
end

if y <= prevActT && y > prevDecayT
    decayT = decSmoothing * actT + (1-decSmoothing)*prevDecayT;
end

if y <= prevDecayT
    decayT = prevDecayT * decayRate;
end

if y > prevFastT
    fire = 1;
else
    fire = 0;
end

end

function [signals,avgFilter,stdFilter] = ThresholdingAlgo(y,lag,threshold,influence)
signals = zeros(length(y),1);
filteredY = y(1:lag+1);
avgFilter(lag+1,1) = mean(y(1:lag+1));
stdFilter(lag+1,1) = std(y(1:lag+1));
for i=lag+2:length(y)
    if abs(y(i)-avgFilter(i-1)) > threshold*stdFilter(i-1)
        if y(i) > avgFilter(i-1)
            signals(i) = 1;
        else
            signals(i) = -1;
        end
        filteredY(i) = influence*y(i)+(1-influence)*filteredY(i-1);
        avgFilter(i) = mean(filteredY(i-lag:i));
        stdFilter(i) = std(filteredY(i-lag:i));
    else
        signals(i) = 0;
        filteredY(i) = y(i);
        avgFilter(i) = mean(filteredY(i-lag:i));
        stdFilter(i) = std(filteredY(i-lag:i));
    end
end
end

%% Demo screen function
function [] = DemoScreen(n,lag,threshold,influence)
figure('Position',[200 100,1000,500]);
subplot(2,1,1);
title(sprintf(['Draw data points (%.0f max)      [settings: lag = %.0f, '...
    'threshold = %.2f, influence = %.2f]'],n,lag,threshold,influence));
ylim([0 5]); xlim([0 100]);
H = gca; subplot(2,1,1);
set(H, 'YLimMode', 'manual'); set(H, 'XLimMode', 'manual');
set(H, 'YLim', get(H,'YLim')); set(H, 'XLim', get(H,'XLim'));
xg = []; yg = [];
for i=1:n
    try
        [xi,yi] = ginput(1);
    catch
        return;
    end
    xg = [xg xi]; yg = [yg yi];
    if i == 1
        subplot(2,1,1); hold on;
        plot(H, xg(i),yg(i),'r.'); 
        text(xg(i),yg(i),num2str(i),'FontSize',7);
    end
    if length(xg) > lag
        [signals,avg,dev] = ...
            ThresholdingAlgo(yg,lag,threshold,influence);
        area(xg(lag+1:end),avg(lag+1:end)+threshold*dev(lag+1:end),...
            'FaceColor',[0.9 0.9 0.9],'EdgeColor','none');
        area(xg(lag+1:end),avg(lag+1:end)-threshold*dev(lag+1:end),...
            'FaceColor',[1 1 1],'EdgeColor','none');
        plot(xg(lag+1:end),avg(lag+1:end),'LineWidth',1,'Color','cyan');
        plot(xg(lag+1:end),avg(lag+1:end)+threshold*dev(lag+1:end),...
            'LineWidth',1,'Color','green');
        plot(xg(lag+1:end),avg(lag+1:end)-threshold*dev(lag+1:end),...
            'LineWidth',1,'Color','green');
        subplot(2,1,2); hold on; title('Signal output');
        stairs(xg(lag+1:end),signals(lag+1:end),'LineWidth',2,'Color','blue');
        ylim([-2 2]); xlim([0 100]); hold off;
    end
    subplot(2,1,1); hold on;
    for j=2:i
        plot(xg([j-1:j]),yg([j-1:j]),'r'); plot(H,xg(j),yg(j),'r.');
        text(xg(j),yg(j),num2str(j),'FontSize',7);
    end
end
end

%% Demo screen function
function [] = ConflictScreen(n)
figure('Position',[200 100,1000,500]);
subplot(2,1,1);
title(sprintf(['Draw data points (%.0f max)'], n));
ylim([0 5]); xlim([0 100]);
H = gca; subplot(2,1,1);
set(H, 'YLimMode', 'manual'); set(H, 'XLimMode', 'manual');
set(H, 'YLim', get(H,'YLim')); set(H, 'XLim', get(H,'XLim'));
xg = []; yg = [];


maxT = 0.04;
fastT = 0.04;
actT = 0.04;
decayT = 0.04;

maxTs = [];
fastTs = [];
actTs = [];
decayTs = [];
fires = [];

for i=1:n
    try
        [xi,yi] = ginput(1);
    catch
        return;
    end
    
    xg = [xg xi]; yg = [yg yi];
    
    if i == 1
        subplot(2,1,1); hold on;
        plot(H, xg(i),yg(i),'r.'); 
        text(xg(i),yg(i),num2str(i),'FontSize',7);
        [fire, maxT, fastT, actT, decayT] = ...
            conflictThreshold(yi, maxT, fastT, actT, decayT);
    end
    
    if i > 1
        [fire, maxT, fastT, actT, decayT] = ...
            conflictThreshold(yi, maxTs(end), fastTs(end), actTs(end), decayT(end));       
    end 

    maxTs = [maxTs maxT];
    fastTs = [fastTs fastT];
    actTs = [actTs actT];
    decayTs = [decayTs decayT];
    fires = [fires fire];    
    
    if i == 1
        continue
    end

    plot(xg(i-1:end),maxTs(i-1:end),'LineWidth',1,'Color','cyan');
    plot(xg(i-1:end),fastTs(i-1:end),'LineWidth',1,'Color','yellow');
    plot(xg(i-1:end),actTs(i-1:end),'LineWidth',1,'Color','green');
    plot(xg(i-1:end),decayTs(i-1:end),'LineWidth',1,'Color','black');
    
    subplot(2,1,2); hold on; title('Signal output');
    stairs(xg(i-1:end),fires(i-1:end),'LineWidth',2,'Color','blue');
    ylim([-2 2]); xlim([0 100]); hold off;
        
    subplot(2,1,1); hold on;
    for j=i:i
        plot(xg([j-1:j]),yg([j-1:j]),'r'); plot(H,xg(j),yg(j),'r.');
        text(xg(j),yg(j),num2str(j),'FontSize',7);
    end
end
end