function [ scores ] = colorize( directory )
% Colorizes everything in the target directory and scores them

CROP_AMOUNT = 20;

files = dir(strcat(directory, '*.png'));
files = {files(~[files.isdir]).name};

scoreFiles = dir(strcat(directory, '*score.txt'));
scoreFiles = {scoreFiles(~[scoreFiles.isdir]).name};

scores = struct;

for i = 1:numel(files)
    fprintf('Processing %d of %d...\n', i, numel(files));
    
    file = char(files(i));
    fileName = char(fullfile(directory, file));
    fieldName = char(strrep(file, '.', '_'));    
    fieldName = strrep(fieldName, '-', 'N');
    
    scores.(fieldName) = struct;
    
    image = imread(fileName);
    
    croppedImage = image(CROP_AMOUNT+1:end-CROP_AMOUNT, ...
                         CROP_AMOUNT+1:end-CROP_AMOUNT, :);
                     
    scores.(fieldName).raw = image;
    scores.(fieldName).cropped = croppedImage;
    
    colorizedImage = coolColors(croppedImage);
    scores.(fieldName).colorized = colorizedImage;
    
    hueImage = imgaussfilt(colorizedImage, 4); % we'll convert back to RGB after getting the hues
    hueImage = rgb2hsv(hueImage);
    hueImage = squeeze(hueImage(:,:,1));
    
    scores.(fieldName).hue = hueImage;
    
    [t, s] = coolScores(croppedImage, false);
    scores.(fieldName).totals = t;
    scores.(fieldName).scores = s;    
    
    f = fopen(fullfile(directory, char(scoreFiles{i})));
    data = textscan(f, '%f');
    scores.(fieldName).learningScore = data{1};
    fclose(f);
end

end

