function [  ] = convert( input_folder, output_folder )
%convert Preprocesses Berkeley BSD dataset images

% convert('/data/datasets/border/berkeley_BSDS500/BSR/BSDS500/data/groundTruth/train', '/data/datasets/border/berkeley_BSDS500/BSR/BSDS500/data/groundTruth/converted')
% convert('/data/datasets/border/berkeley_BSDS500/BSR/BSDS500/data/groundTruth/test', '/data/datasets/border/berkeley_BSDS500/BSR/BSDS500/data/groundTruth/converted')
% convert('/data/datasets/border/berkeley_BSDS500/BSR/BSDS500/data/groundTruth/val', '/data/datasets/border/berkeley_BSDS500/BSR/BSDS500/data/groundTruth/converted')

files = dir(strcat(input_folder, '/*.mat'));
for file = files'
    mat = load(strcat(input_folder, '/', file.name));
    
    % choose version with smallest number of segments
    smallest = 99999;
    smallest_index = 1;
    for i = 1:length(mat.groundTruth)
        count = max(max(mat.groundTruth{i}.Segmentation));
        if count < smallest
            smallest = count;
            smallest_index = i;
        end
    end    
    
    boundaries = mat.groundTruth{smallest_index}.Boundaries;
    
    % remove all small segments    
    segments   = mat.groundTruth{smallest_index}.Segmentation;
%     for i = 1:max(max(segments))
%         if(sum(sum(segments == i)) < 1000)
%             mask = bwmorph(segments == i, 'dilate');
%             boundaries(mask) = 0;
%         end
%     end
    
    % resize and binarize
    img = imresize(double(boundaries), 1, 'Antialiasing', true);
    img2 = img > 0.05;
%     imshow(boundaries);
%     img2 = double(boundaries);
    
    [~, name, ~] = fileparts(file.name);
    
    out_name = strcat(output_folder, '/', name, '.png');
    imwrite(img2, out_name);
    
    % make 4 connected, uses Image Magick
    command = horzcat('convert ', out_name, ' -morphology thicken ''3>:-,0,-,1,-,1,0,1,0'' ', ...
        '-morphology thicken ''3>:-,0,0,1,-,0,0,1,-'' ', ...
        '-morphology thinning Skeleton:3 ', ...
        out_name);
    
    system(command);    
    
    % Blur final result
    x = imread(out_name);
    x = double(x);
    y = mat2gray(imgaussfilt(x, 1));
    imwrite(y, out_name);
    
%     imshow(imread(out_name));
end


end

