function [ out ] = coolColors( image, RADIUS, MAX_PTS, THRESHOLD )
%coolColors Takes sparse output from C++ program and interpolates to fit a
%dense grid

% default values: r = 6, max = 4, thresh = 0
% less black stuff: 6, 4, 20

if nargin < 2
    RADIUS = 4;
    MAX_PTS = 20;
    THRESHOLD = 20;
end

image = rgb2lab(double(image) / 255);
out = zeros(size(image));

jSize = size(image, 2);

parfor i = 1:size(image, 1)
%     i
    for j = 1:jSize
        if image(i, j, 1) > THRESHOLD
            out(i, j, :) = image(i, j, :);
        else
            out(i, j, :) = localSearch( image, j, i, RADIUS, MAX_PTS, THRESHOLD );
        end
    end
end

out = lab2rgb(out);
end

function color = localSearch( image, x, y, radius, MAX_PTS, THRESHOLD )

x_min = max(1, x-radius);
y_min = max(1, y-radius);
x_max = min(size(image, 2), x+radius);
y_max = min(size(image, 1), y+radius);

points = zeros(sum(sum(image(y_min:y_max, x_min:x_max, 1) > THRESHOLD)), 3);
index = 1;

numPts = min(size(points, 1), MAX_PTS);
if numPts < 1
    color = zeros(1, 1, 3);
    return
end

for i = x_min:x_max
    for j = y_min:y_max
        if image(j, i, 1) > THRESHOLD
            points(index, :) = [j, i, max(pdist([j, i; y, x]), 1)];
            index = index + 1;
        end
    end
end

sorted = sortrows(points, 3);

dists = 1 ./ (sorted(1:numPts, 3) .* sorted(1:numPts, 3));
% dists = 1 ./ (sorted(1:numPts, 3));
dists = dists / sum(dists);

color = zeros(1, 1, 3);

for i = 1:numPts
    c = image(sorted(i, 1), sorted(i, 2), :);
    color = color + double(c) * dists(i);
end
% color = lab2rgb(color);
% color = uint8(round(color));

end