function [] = makeAllMedianComparisonPlot( directory )
%makeAllMedianComparisonPlot Makes bar chart showing all medians
%   Pass this the raw scores directory, which should contain
%   subdirectories with names like cl_all, cl_unlearn_only, hebb, etc
%   Each of those should have raw polarity score files like
%   2_polarity_scores.txt

% Ordering
% hebb
% cl_spreading_only
% cl_unlearn_only
% cl_slt_only
% cl_no_spreading
% cl_no_unlearn
% cl_no_slt
% cl_all

CONF_INTERVAL = 5;

scores = [loadScores( directory, 'hebb' );
          loadScores( directory, 'cl_spreading_only' );
          loadScores( directory, 'cl_unlearn_only' );
          loadScores( directory, 'cl_slt_only' );
          loadScores( directory, 'cl_no_spreading' );
          loadScores( directory, 'cl_no_unlearn' );
          loadScores( directory, 'cl_no_slt' );
          loadScores( directory, 'cl_all' )];
      
medians = median(scores,2);
lower = medians - prctile(scores, 50 - CONF_INTERVAL, 2);
upper = prctile(scores, 50 + CONF_INTERVAL, 2) - medians;

errors = lower; errors(:,:,end+1) = upper;

barHandle = barwitherr(errors, medians);

color_map = [0, 0, 1; 1, 0, 0];
colormap(color_map);

pValues = zeros(1, 8);
for i = 1:7
    pValues(i) = ranksum(scores(i,:), scores(end,:));
end
end

function S = loadScores( directory, name )

POLARITY_FILE = '2_polarity_scores.txt';
filename = sprintf('%s/%s/%s', directory, name, POLARITY_FILE);
S = dlmread(filename)';

end