function [ hebbScores, standardScores ] = makeComparisonPlots( newScores )
%makeComparisonPlots Creates plots of input and responses for hebbian and
%standard

%% Separate scores by type and bias
cellArr = struct2cell(newScores);
Acell = reshape(cellArr, size(cellArr, 1), []);
Acell = Acell';
Acell = sortrows(Acell, -2);

hebbScores = {};
standardScores = {};
for i = 1:size(Acell, 1)
    if strcmp(Acell(i, 1), 'hebbian')
        hebbScores{end+1} = squeeze(Acell(i, :));
    else
        standardScores{end+1} = squeeze(Acell(i, :));
    end
end

hebbSize = [size(hebbScores{1}, 2), size(hebbScores, 2)];
standardSize = [size(standardScores{1}, 2), size(standardScores, 2)];

hebbScores = [hebbScores{:}];
hebbScores = reshape(hebbScores, hebbSize);

standardScores = [standardScores{:}];
standardScores = reshape(standardScores, standardSize);

isNoise = max(cell2mat(hebbScores(2,:))) == 0.7;

if isNoise
    hebbScores = fliplr(hebbScores);
    standardScores = fliplr(standardScores);
end

%% Make plots
figure;
numRows = size(hebbScores, 2); %-1 is for ignoring natural images
numCols = 4;
subplotMargins = [0.02, 0.03];
for r = 1:numRows
    idxShift = numCols * (r - 1);
    
    % Comment to use natural distribution
    if( hebbScores{2, r} < 0 )
        continue;
    end
    
    % Bias plot
    subplot_tight(numRows, numCols, idxShift + 1, subplotMargins);
    makeBiasPlot( hebbScores{2, r} );
    
    % Hebbian colors map
    subplot_tight(numRows, numCols, idxShift + 2, subplotMargins);
    imshow(normalizeRGB(hebbScores{3, r}.colorized));
    
    % Standard colors map
    subplot_tight(numRows, numCols, idxShift + 3, subplotMargins);
    imshow(normalizeRGB(standardScores{3, r}.colorized));
    
    % Orientation distribution plot
    subplot_tight(numRows, numCols, idxShift + 4, subplotMargins);
    hold on
    plot(hebbScores{3, r}.totals);
    plot(standardScores{3, r}.totals);
    hold off
    
end

%% Make plot of scoring metric
figure;
hold on

sHebbian       = zeros(numRows, 1);
sStandard      = zeros(numRows, 1);

for i = 1:numRows
    sHebbian(i) = hebbScores{3, i}.scores;
    sStandard(i) = standardScores{3, i}.scores;
end

plot(sHebbian);
plot(sStandard);

hold off

end

function makeBiasPlot( bias )
vals = cool_rand( pi/2, bias, 0, pi, 100000);
hist(vals, 100);
xlim([0, pi]);
ylim([0, 4000]);

end

