function [ x, y, dx, dy ] = makeV2QuiverPlot( directory, prefix )
%makeV2QuiverPlot Makes a quiver plot from Border Ownership output
%   This can be further visualized by running the python script
%   quiver.pyir
%
%   Input should be the txt files generated when clicking on a BO neuron.
files = dir(strcat(directory, strcat(prefix, '*.txt')));
files = {files(~[files.isdir]).name};

% will load files in counter-clockwise order starting with 0 degrees
% down, up
% btm right, upr left,
% right, left
% upr right, dwn left

A = []; % for activations

for f = 1:numel(files)    
    file = char(files(f));
    
    path = char(fullfile(directory, file)); 
    D = dlmread(path);
    
    if ~isempty(A)
        A(:,:,size(A, 3)+1) = D;    
    else
        A = D;
    end
end

Q = zeros(size(A,1), size(A,2), 2); % holds quiver vectors
for i = 1:size(A,1)
    for j = 1:size(A,2)
        for k = 1:size(A,3)            
            angle = floor((k-1)/2) * 45 - 90;
            if mod(k,2) == 0
                angle = angle + 180;
            end
            
            
            vec = [cosd(angle); sind(angle)];            
            vec = reshape(vec, size(Q(i,j,:)));
            
%             if i == 24 && j == 19
%                 vec
%                 A(i,j,k)
%                 angle                
%             end
            
            Q(i,j,:) = Q(i,j,:) + vec * A(i,j,k);     
%             fprintf('%d %d %d %f %f\n', i, j, k, vec[0], vec[1]);
        end        
    end
end

% flip y axis because image coordinates
Q = flipud(Q);


[x, y] = meshgrid(1:size(A,2), 1:size(A,1));
dx = squeeze(Q(:,:,1));
dy = squeeze(Q(:,:,2));

% save mat
save('quiver.mat', 'x', 'y', 'dx', 'dy');

end

