function [ barHandle ] = plotStatisticalError( data1, data2 )
% data1 and data2 are n x k matrices where n is number of samples and
% k is number of features.
n = size(data1, 1);
k = size(data1, 2);
means1 = mean(data1);
means2 = mean(data2);
std1 = std(data1);
std2 = std(data2);

pValues = zeros(1, k);
for i = 1:k
    pValues(i) = ttest2(data1(:, i), data2(:, i));
end

barHandle = barwitherr([std1' std2'], [means1' means2']);
for i = 1:k
    if pValues(i)
        barBuffer = 0.05;
        height = max( means1(i) + std1(i), means2(i) + std2(i) ) + barBuffer;
        plotSigStar(barHandle, i, height);
    end
end
end

function plotSigStar(barHandle, k, height)
ctr1 = bsxfun(@plus, barHandle(1).XData, [barHandle(1).XOffset]');
ctr2 = bsxfun(@plus, barHandle(2).XData, [barHandle(2).XOffset]');
hold on
centers = [ctr1(k) ctr2(k)];
plot(centers, [1 1]*height, '-k', 'LineWidth',2)
plot(mean(centers), height + 0.05, '*k')
hold off
end