'''
Loads a mat file created in matlab with makeV2QuiverPlot and generates
nice looking graphics for it.  Expects to find the following variables:
x, y, dx, dy
'''

import sys
import scipy.io as sio
from pylab import *
from numpy import ma

import matplotlib as mpl
import matplotlib.cm as cm

# First arg will be the mat file
mat = sys.argv[1]
mat_contents = sio.loadmat(mat);

# scale everything by second arg
scale = float(sys.argv[2])

X = mat_contents['x'];
Y = mat_contents['y'];
U = mat_contents['dx'];
V = mat_contents['dy'];

# Scaling
U = U * scale
V = V * scale

# Generate color map
# get colors with m.to_rgba(xxx)
C = arctan2(U,V)
       # +pi) / (2*pi)
np.set_printoptions(threshold=np.nan)
#print C

#2
figure()
M = zeros(U.shape, dtype='bool')
M[logical_and(U == 0, V == 0)] = True
U = ma.masked_array(U, mask=M)
V = ma.masked_array(V, mask=M)
Q = quiver( X, Y, U, V, C, angles='xy', scale_units='xy', scale=2,
            color='r', units='x',
            cmap='hsv',
            width=.2,
            linewidths=(.05,),
            edgecolors=('k'),
            headaxislength=3,
            headwidth=4)
#, scale=100, linewidths=(2,) )


# square overlap
#axis([13,33, 8,28])
# c shape
#axis([6,45, 12,34])

axis('equal')

# optionally save file
if len(sys.argv) > 3:
    axis('off')
    gca().set_position([0, 0, 1, 1])
    savefig(sys.argv[3])
else:
    show()
