'''
Performs the entire process of generating nice looking graphics for
a border ownership assignment. See also quiver.py which works in tandem
with the matlab makeV2QuiverPlot.
'''

import sys
import scipy.io as sio
from pylab import *
import numpy as np
from numpy import ma

import matplotlib as mpl
import matplotlib.cm as cm

import glob

# Get directory from first arg
directory = sys.argv[1]
# Get prefix from second arg
prefix = sys.argv[2]

files = sorted(glob.glob(directory + prefix + '*.txt'))

Activations = []

# Load the activations in counter-clockwise order starting from 0 degrees
#   down, up
#   btm right, upr left
#   right, left
#   upr right, dwn left
for f in files:
    D = np.loadtxt(f)
    Activations.append(D)
Activations = np.array(Activations)
Activations = np.swapaxes(Activations,0,1)
Activations = np.swapaxes(Activations,1,2)


# Build quiver vectors
Q = np.zeros([Activations.shape[0], Activations.shape[1], 2])
print Q.shape
for i in range(Activations.shape[0]):
    for j in range(Activations.shape[1]):
        for k in range(Activations.shape[2]):
            angle = math.floor((k-1)/2.0) * 45 - 90
            if k % 2 == 0:
                angle = angle + 180

            angle = np.radians(angle)

            vec = np.array([np.cos(angle), sin(angle)])

            Q[i,j,:] = Q[i,j,:] + vec * Activations[i,j,k]
            print i, j, k, vec

sio.savemat('test.mat', mdict={'snarf':Q})

# Flip along y axis for image coordinates
np.flipud(Q)

print Activations.shape
[X, Y] = np.meshgrid(np.arange(1, Activations.shape[1] + 1), np.arange(1, Activations.shape[0] + 1))
U = np.squeeze(Q[:,:,0])
V = np.squeeze(Q[:,:,1])

print X.shape, Y.shape, U.shape, V.shape

# Generate color map
# get colors with m.to_rgba(xxx)
C = arctan2(U,V)
       # +pi) / (2*pi)
np.set_printoptions(threshold=np.nan)
#print C

#2
figure()
M = zeros(U.shape, dtype='bool')
M[logical_and(U == 0, V == 0)] = True
U = ma.masked_array(U, mask=M)
V = ma.masked_array(V, mask=M)
Q = quiver( X, Y, U, V, C, angles='xy', scale_units='xy', scale=2,
            color='r', units='x',
            cmap='hsv',
            width=.2,
            linewidths=(.05,),
            edgecolors=('k'),
            headaxislength=3,
            headwidth=4)
#, scale=100, linewidths=(2,) )


# square overlap
#axis([13,33, 8,28])
# c shape
#axis([6,45, 12,34])

show()
