function [ newIm ] = resizeImageToBoundary( mask, r, minSize )
% Assumes im is a binary matrix

% Get centroid
centroid = getCentroid(mask);

% Get average distance from CoM
edgesMask = getEdges(mask);

numEdges = sum(sum(edgesMask));
aveDist = 0;
for i = 1:size(edgesMask, 1)
    for j = 1:size(edgesMask, 2)
        if edgesMask(i, j)
            aveDist = aveDist + pdist([centroid; i, j]);
        end
    end
end

dist = aveDist / numEdges;

% [~, BW] = max_inscribed_circle(edgesMask*255, 0);
% BW(logical(mask)) = 0;
% dist = max(max(BW));

% shrink
newIm = imresize(double(mask), r / dist);
newIm = double(~(newIm > 0.3));
% edges
newIm = getEdges( newIm );
newIm = pad(newIm, minSize);

end

function centroid = getCentroid(objectMask)
    xSum = sum(objectMask, 2);
    ySum = sum(objectMask, 1);
    total = sum(xSum);
    xAve = xSum' * (1:numel(xSum))' / total;
    yAve = ySum * (1:numel(ySum))' / total;
    centroid = [xAve, yAve];
end

function [ m ] = getEdges( m )
% m = conv2(m, ones(3), 'same');
% m = (m ~= 0 & m ~= 9);
% m = double(m);

m = imfilter(m, ones(2), 0, 'same');
m = (m ~= 0 & m ~= 4);
m = double(m);
end

function m_final = pad( m_old, minSize )
m = zeros(size(m_old) + [1 1]);
m(2:end,2:end) = m_old;

padH = max(floor((minSize(1) - size(m, 1)) / 2), 0);
padW = max(floor((minSize(2) - size(m, 2)) / 2), 0);

m = padarray(m, [padH, padW]);

% take care of the case where we are one off from minSize
h_off = minSize(1) - size(m, 1);
w_off = minSize(2) - size(m, 2);
m_final = zeros(size(m) + [h_off, w_off]);
m_final(1+h_off:end,1+w_off:end) = m;


end