function [ h_results, s_results ] = v1_stability( directory, bias_val, network_idx_start, network_idx_stop, iter_start, iter_step, iter_max, type )
%v1_stability Computes stability over a range of v1 results
% type is either 'bias' or 'noise'
% bias_val = 0.5;
h_results = zeros(2, numel(iter_start:iter_step:iter_max));
s_results = zeros(2, numel(iter_start:iter_step:iter_max));
i = 1;
for iter = iter_start:iter_step:iter_max
    vals = [];
    for network_idx = network_idx_start:network_idx_stop
        value = GetValue(directory, iter, iter_max, network_idx, 'h', bias_val, type);
        if value ~= -10
            vals = [vals value];
        end
    end
    h_results(1,i) = mean(vals);
    h_results(2,i) = std(vals);
    
    vals = [];
    for network_idx = network_idx_start:network_idx_stop
        value = GetValue(directory, iter, iter_max, network_idx, 's', bias_val, type);
        if value ~= -10
            vals = [vals value];
        end
    end
    s_results(1,i) = mean(vals);
    s_results(2,i) = std(vals);
    
    i = i + 1;
end
end

function value = GetValue(directory, iter, iter_max, network_idx, typeLetter, bias, type)
    name = sprintf('iteration_%d_%d_%s.mat', network_idx, iter_max, type);
    load(char(fullfile(directory, name))); % will load into variable 'scores'
    max_data = GetDamnData(scores, typeLetter, bias);

    name = sprintf('iteration_%d_%d_%s.mat', network_idx, iter, type);
    load(char(fullfile(directory, name))); % will load into variable 'scores'
    iter_data = GetDamnData(scores, typeLetter, bias);

    if isstruct(max_data) && isstruct(iter_data)
        value = v1_similarity(iter_data.colorized, max_data.colorized);
    else
        value = -10;
    end
end

function data = GetDamnData(scores, typeLetter, bias)
    data = -1;
    for i = 1:numel(scores)
        if (scores(i).bias == bias && scores(i).type(1) == typeLetter)
            data = scores(i).data;
        end
    end
end