#include <engine.h>
#include <iostream>
#include <cstring>
#include "GaborFit.H"

struct EngineWrapper
{
  EngineWrapper() :
    //engine( engOpen("matlab_local") )
    engine( engOpen("") )
  {
    // Set up path to library
    engEvalString( engine, "D = pwd" );
    engEvalString( engine, "cd /lab/shane/shane/projects/Border/matlab" );
    engEvalString( engine, "addpath(genpath(pwd))" );
    engEvalString( engine, "cd /lab/shane/workspace/knkutils" );
    engEvalString( engine, "addpath(genpath(pwd))" );
    engEvalString( engine, "cd(D)" );
  }

  ~EngineWrapper()
  {
    engClose( engine );
  }

  Engine * engine;
};


std::pair<double, double> matlab::evaluateAngle( double * data, int width, int height )
{
  static thread_local EngineWrapper EW;

  // Copy data to MATLAB
  mxArray * T = mxCreateDoubleMatrix( height, width, mxREAL );
  std::memcpy( mxGetPr( T ), data, sizeof(double) * width * height );
  engPutVariable( EW.engine, "T", T );

  //static int xxx = 0; // DEBUG

  // Get result
  engEvalString( EW.engine, "T = T / sum(sum(abs(T)));" ); // normalize
  engEvalString( EW.engine, "[params, R, exit] = fitgabor2d(T);" );
  engEvalString( EW.engine, "theta = params(2) * 180.0 / pi;" );

  // DEBUG
  //std::string bla = "save('/lab/shane/shane/projects/Border/build_n/" + std::to_string(xxx) + ".mat', 'T');";
  //engEvalString( EW.engine, bla.c_str() );

  mxArray * result   = engGetVariable( EW.engine, "theta" );
  mxArray * R = engGetVariable( EW.engine, "R" );

  // DEBUG
  //std::cout << xxx << " ........... " << std::endl;
  //++xxx;

  const double theta = mxGetScalar( result );
  const double error = mxGetScalar( R );

  mxDestroyArray( T );
  mxDestroyArray( result );
  mxDestroyArray( R );

  return {theta, error};
}

void matlab::evaluateScores( std::string const & directory, std::string const & outputName )
{
  static thread_local EngineWrapper EW;

  const std::string command = "scores = colorize('" + directory + "'); "
                              "scores = reformatScoresStruct(scores); "
                              "save('" + outputName + "', 'scores'); ";

  engEvalString( EW.engine, command.c_str() );
}

void matlab::generatePinwheelsInputImage( std::string const & inputImage, std::string const & outputName )
{
  static thread_local EngineWrapper EW;

  const std::string command = "x = imread('" + inputImage + "'); "
                              "makePinwheelMat(x, '" + outputName + "');";

  engEvalString( EW.engine, command.c_str() );
}
