#include <utility>
#include <string>

namespace matlab
{
  //! Fits a gabor to the given data and returns the angle in degrees and the R2 error
  /*! Data should be in MATLAB friendly layout (column major)
      0 degrees is to the right, positive counter clockwise */
  std::pair<double, double> evaluateAngle( double * data, int width, int height );

  //! Evaluates all scores from within MATLAB for all images in a directory
  void evaluateScores( std::string const & directory, std::string const & outputName );

  //! Generates the input for the pinwheels analysis from a raw V1 image
  /*! Generated image will be stored in a mat file with the specified name. To be
      analyzed by the python code for pinwheel density.
      @param outputName the name of the mat file */
  void generatePinwheelsInputImage( std::string const & inputImage, std::string const & outputName );
}
