#!/usr/bin/env python
"""
Portions of this file:

Copyright (c) 2005-2012, Topographica Developers
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

 * Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

 * Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the
   distribution.

 * Neither the name of Topographica nor the names of its contributors
   may be used to endorse or promote products derived from this
   software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""
from optparse import OptionParser
import sys
import analysis
import matplotlib.pyplot as plt
import numpy as np
from matplotlib.colors import hsv_to_rgb

def pinwheel_overlay(pinwheels, image=None, contours=None, style='wo',linewidth=1):
   """
   Plots the pinwheel locations and optionally the real and imaginary
   pinwheel contours. Designed to be overlayed over an OR map.
   """
   fig = plt.figure(frameon=False)


   fig.patch.set_alpha(0.0)
   ax = plt.subplot(111, aspect='equal', frameon=True)
   ax.patch.set_alpha(0.0)
   if image is not None:
       #plt.imshow(image)
       hsv = np.dstack((image, np.ones_like(image), np.ones_like(image)))
       rgb = hsv_to_rgb(hsv)
       plt.imshow(rgb)
   plt.hold(True)

   scaling = 1.0 if image is None else image.shape[0]

   (recontours, imcontours) = contours if contours else ([],[])
   #for recontour in recontours:
   #    plt.plot(recontour[:,0] * scaling, recontour[:,1] * scaling,'k',linewidth=linewidth)
   #for imcontour in imcontours:
   #    plt.plot(imcontour[:,0] * scaling, imcontour[:,1] * scaling,'w', linewidth=linewidth)

   if pinwheels is not None:
      Xs, Ys = zip(*pinwheels)
      plt.plot(np.array(Xs) * scaling, np.array(Ys) * scaling, style)

   limit = (0, scaling) if image is not None else (0.0,1.0)

   plt.xlim(limit);             plt.ylim(limit)
   ax.xaxis.set_ticks([]);      ax.yaxis.set_ticks([])
   ax.xaxis.set_ticklabels([]); ax.yaxis.set_ticklabels([])
   return fig

###############

# Set up options
parser = OptionParser()
parser.add_option("--mat", help="MAT file containing the image data (2D matrix of hue values stored as 'hueImage', created using makePinwheelMat.m)", default="xxx.mat")

# Collect options
(options, args) = parser.parse_args(sys.argv)

# Load mat file
image = analysis.matToImage(str(options.mat))

results = analysis.pinwheel_analysis(image)

pinwheel_plt = pinwheel_overlay(results['pinwheels'], image, (results['re_contours'], results['im_contours']))
plt.show()

savefig_opts = dict(transparent=True, format='svg', bbox_inches='tight', pad_inches=0)
pinwheel_plt.savefig('overlay.svg', **savefig_opts)

#print results['metadata']

pinwheel_count = len(results['pinwheels'])
kmax = results['metadata']['kmax']
pinwheel_density = pinwheel_count / (kmax**2)

print 'Pinwheel count:', pinwheel_count, 'kmax:', kmax, 'density:', pinwheel_density
