# A Runs the V1 experiment multiple times, saving processed
# matlab images to disk.  cereal files for all but the last run
# are destroyed.
#
# grab the number of iterations from command line $1
# grab the image save step from command line $2
# grab the image generation number from command line $3

if [ "$#" -ne 3 ]; then
  echo "Please specify the number of iterations, save step, and image iteration number as the only parameters"
fi

for i in `seq 1 $1`;
do
  echo "On iteration $i"
 
  # remove old files and do a run
  rm *.cereal;
  ./edge ../config/amd_edge.json 3 3 3 3

  # iterate over each save 
  for j in `seq $2 $2 $3`;
  do
    # generate images for a run
    pattern="./*_${j}_v1_*.cereal"
    echo "Working on pattern: $pattern"
   
    for f in $pattern;
    do
      ./edge ../config/amd_edge.json $f
    done
  
    # generate final analysis for this run
    #echo "before"
    #ls
    mkdir temp_noise
    mv *noise*.png temp_noise
    mv *noise*_score.txt temp_noise

    #echo "after noise"
    #ls

    mkdir temp_bias
    mv *.png temp_bias
    mv *_score.txt temp_bias

    #echo "after bias"
    #ls

    input="$(pwd)/temp_bias/"
    output="$(pwd)/iteration_${i}_${j}_bias.mat"
    ./evaluateScores $input $output
    
    input="$(pwd)/temp_noise/"
    output="$(pwd)/iteration_${i}_${j}_noise.mat"
    ./evaluateScores $input $output

    rm *.png
    rm -rf temp_noise
    rm -rf temp_bias
  done
done
