#ifndef BORDEROWNERSHIPNEURON_H_
#define BORDEROWNERSHIPNEURON_H_

#include "Neuron.H"
#include <opencv2/core/core.hpp>

namespace bd
{
  //! A neuron that decides border ownership
  /*! This neuron is always created in pairs with a
      built in inhibitory connection to its sibling such
      that only one of them fires at stable state */
  class BorderOwnershipNeuron : public Neuron
  {
    public:
      using Neuron::Neuron; // inherit ctors

      template <class Archive>
      void serialize( Archive & ar )
      {
        ar( cereal::base_class<Neuron>( this ),
            orientation );
      }

      //! The orientation this neuron receives input from
      /*! This is used for visualization and not for any computation */
      double orientation;

    protected:
      virtual double updateImpl( SystemState const & state ) override;

      virtual void learnImpl( SystemState const & state, double currentActivation ) override;
  };
}

#endif // BORDEROWNERSHIPNEURON_H_
