#ifndef COMBINED_H_
#define COMBINED_H_

#include <opencv2/core/core.hpp>

#include "Gabor.H"
#include "Sheet.H"
#include "BOSheet.H"
#include "State.H"

#include "StaticNeuron.H"
#include "BorderOwnershipNeuron.H"
#include "GroupingNeuron.H"
#include "EdgeNeuron.H"

namespace bd
{
  //! Overall border ownership class
  class Combined
  {
    public:
      /*! @param numScales Number of scales used in V1 Gabor responses
          @param numOrientations Number of edge orientations, used to populate V1 Gabor responses
          @param numGroupingScales Number of scales used in grouping neuron responses
          @param gridWidth Width of simulation (think input pixels)
          @param gridHeight Height of simulation */
      Combined( size_t numBOPairs = 4,
                size_t gridWidth  = 40,
                size_t gridHeight = 40 );

      //! Present a new image for the first time (recomputes v1)
      void onset( cv::Mat const & image );

      //! Advances the network
      void iterate( int numIters = 1 );

      template <class Archive>
      void serialize( Archive & ar )
      {
        ar( CEREAL_NVP(itsState),
            CEREAL_NVP(itsNumBOPairs),
            CEREAL_NVP(itsGridWidth),
            CEREAL_NVP(itsGridHeight),
            CEREAL_NVP(itsEdgeResponses),
            CEREAL_NVP(itsBOResponses),
            CEREAL_NVP(itsGNResponses) );
      }

    public:
    //private:
      SystemState itsState;

      size_t itsNumBOPairs;
      size_t itsGridWidth;
      size_t itsGridHeight;
      size_t itsV1NumScales;
      size_t itsGroupingNumScales;

      //! Input layer
      Sheet<StaticNeuron> itsOnOffResponses;
      Sheet<StaticNeuron> itsOffOnResponses;

      //! V1 layer
      Sheet<EdgeNeuron>  itsEdgeResponses;

      // V2 layers
      //! BO responses, one sheet per two neurons at each location, organized as [orientation][scale] (scale not used)
      std::vector<std::vector<BOSheet>> itsBOResponses;
      //! Grouping responses, organized as [grouping scale] (scale not used)
      std::vector<Sheet<GroupingNeuron>> itsGNResponses;
  };
}

#endif // COMBINED_H_
