#ifndef EDGENEURON_H_
#define EDGENEURON_H_

#include "Neuron.H"
#include <opencv2/core/core.hpp>

namespace bd
{
  template <class N> class Sheet;

  //! A neuron that decides edges
  class EdgeNeuron : public Neuron
  {
    public:
      EdgeNeuron();

      //! Create at position
      EdgeNeuron( std::pair<double, double> pos );

      //! Copy at position
      EdgeNeuron( EdgeNeuron const & other, std::pair<double, double> pos );

      //! Used to ensure FF activation is the same regardless of positioning (think edges)
      double itsFFNormalizationCoefficient;
      double itsLateralNormalizationCoefficient;
      double averageActivation = 0.0;

      template <class Archive>
      void serialize( Archive & ar )
      {
        ar( cereal::base_class<Neuron>( this ) );
        ar( itsFFNormalizationCoefficient );
        ar( itsLateralNormalizationCoefficient );
        ar( averageActivation );
      }

      //! Keeps accumulated counts properly balanced
      /*! This first computes the target maximum accumulated weight, which is
          the number of connections multiplied by the Input::MAX_ACC_WEIGHT parameter.

          If any accumulator has exceeded this maximum, the residual amount will be
          subtraced from all accumulators which will then be divisively normalized.

          @return The target maximum accumulated weight */
      void propogateAccumulatedWeight( SystemState const & state );

      std::vector<double> itsDisplayCache;

    protected:
      virtual double updateImpl( SystemState const & state ) override;

      virtual void learnImpl( SystemState const & state, double currentActivation ) override;
  };
}

#endif // EDGENEURON_H_
