#ifndef NEURON_ALLOCATOR_IMPL_H_
#define NEURON_ALLOCATOR_IMPL_H_

template <class T> inline
NeuronAllocator<T>::NeuronAllocator() noexcept
{ }

template <class T> inline
NeuronAllocator<T>::NeuronAllocator( NeuronAllocator const & other ) noexcept :
  std::allocator<T>( other )
{ }

template <class T>
template <class U> inline
NeuronAllocator<T>::NeuronAllocator( NeuronAllocator<U> const & other ) noexcept
{ }

template <class T> inline
NeuronAllocator<T>::~NeuronAllocator() noexcept
{ }

template <class T> inline
auto NeuronAllocator<T>::allocate( size_type n, const void * ) -> pointer
{
	if ( n > this->max_size() )
	  std::__throw_bad_alloc();

  std::lock_guard<std::mutex> lock( itsMutex );

  auto ptr = reinterpret_cast<pointer>( itsMemory.data() + itsLastIndex );
  itsLastIndex += n * sizeof(T);

  if( itsLastIndex >= itsMaxSize ) // out of space, make that number bigger
    std::__throw_bad_alloc();

  return ptr;
}

template <class T> inline
void NeuronAllocator<T>::deallocate( pointer, size_type )
{
  // do nothing
}

#endif // NEURON_ALLOCATOR_IMPL_H_
