#include "Random.H"
#include <boost/math/distributions/normal.hpp>

auto bd::normalReal( double mean, double variance ) -> std::function<double()>
{
  return [=]()
  {
    static std::random_device rd;
    static std::mt19937 gen(rd());
    std::normal_distribution<double> rng( mean, variance );
    return rng(gen);
  };
}

auto bd::boundedNormalReal( double mean, double std,
                            double lower, double upper ) -> std::function<double()>
{
  return [=]()
  {
    boost::math::normal norm( mean, std );

    auto r = uniformReal( 0.0, 1.0 )();

    const double tail = boost::math::cdf( norm, lower );
    const double head = boost::math::cdf( norm, upper );
    const double remainder = head - tail;

    const double prob = tail + remainder * r;

    //std::cout << "m: " << mean << " std: " << std << " [" << lower << ", " << upper << "]" << std::endl;
    //std::cout << "     r: " << r << std::endl;
    //std::cout << "     t: " << tail << ", h: " << head << ", r: " << remainder << ", p: " << prob << std::endl;

    return boost::math::quantile( boost::math::complement(norm, prob) );
  };
}

