#include <random>

template <class T>
auto bd::uniformInt( T lower, T upper ) -> std::function<T()>
{
  if( upper < lower )
    throw "Upper bound less than lower bound";

  return [=]()
  {
    static std::random_device rd;
    static std::mt19937 gen(rd());
    std::uniform_int_distribution<T> rng( lower, upper );
    return rng(gen);
  };
}

template <class T>
auto bd::uniformReal( T lower, T upper ) -> std::function<T()>
{
  if( upper < lower )
    throw "Upper bound less than lower bound";

  return [=]()
  {
    static std::random_device rd;
    static std::mt19937 gen(rd());
    std::uniform_real_distribution<> rng( lower, upper );
    return rng(gen);
  };
}
