#ifndef THREADING_UTIL_H_
#define THREADING_UTIL_H_

#ifdef WITH_OPENMP
#include <omp.h>
#else
#include <tbb/parallel_for.h>
#endif

#include <cstddef>
#include <iostream>

namespace threading
{
  extern int nthreads;

  template <class Func>
  void parallel_for( const size_t start, const size_t stop, Func && f )
  {
    #ifdef WITH_OPENMP
    #pragma omp parallel num_threads(nthreads)
        {
    #pragma omp for
          for( int i = (int)start; i < (int)stop; ++i )
          {
            f(i);
          }
        }
    #else // on intel
        tbb::parallel_for( start, stop, std::forward<Func>(f) );
        return;
    #endif
  }
}

#endif // THREADING_UTIL_H_
