#ifndef UTIL_H_
#define UTIL_H_

//#include <nrt/Core/Image/Image.H>
#include <opencv2/core/core.hpp>

namespace bd
{
  //cv::Mat toMat( nrt::Image<nrt::PixGray<float>> const & source );

	//! Swap the quadrants in a matrix in place
	/*! Has the effect of centering something at (0, 0)
	 		@param[in] src The input matrix
	 		@param[out] dest The output matrix. */
	void fftshift( cv::Mat & src );

	//! Undoes an fftshift
	/*! @param[in] src The input matrix
	 		@param[out] dest The output matrix. */
	void ifftshift( cv::Mat & src );

	//! Calculates the median of a matrix
	float median( cv::Mat const & matrix );

	//! Prepares an image for FFT
	/*! Pads an image by duplicating borders and adding zero valued complex data.  Should be used
			for FFT based convolution where the filters are created in the frequency domain
			@param[in] input The input image
			@param[in] paddedSize The size of the padded image
			@return The padded image, complex valued */
	cv::Mat padImageFFT( cv::Mat const & input, cv::Size const & paddedSize );

	//! Prepares an image for FFT
	/*! Pads an image by duplicating borders in such a way to support cyclic convolution
	 		Adds zero valued complex data.  Should be used with filters initially created in spatial domain.
			@param[in] input The input image
			@param[in] kernelSize size of largest unpadded kernel
			@param[in] paddedSize The size of the padded image
			@return The padded image, complex valued */
	cv::Mat padImageFFT( cv::Mat const & input, cv::Size const & kernelSize, cv::Size const & paddedSize );

	//! Prepare a kernel for FFT
	/*! The kernel is adjusted such that its center is at (0,0) and filled with zeros appropriately
	 		@param[in] input the unpadded kernel, real or complex valued
			@param[in] paddedSize the size of the padded kernel
			@return The padded kernel, complex valued */
	cv::Mat padKernelFFT( cv::Mat const & input, cv::Size const & paddedSize );

	//! Gets the padded size for an FFT convolution
	cv::Size getPaddedSize( cv::Size const & imageSize );

	//! Gets the padded size for an FFT convolution
	cv::Size getPaddedSize( cv::Size const & imageSize, cv::Size const & largestKernelSize );

	//! Uncrops the result of an FFT, removing the complex channel and normalizing by
	//! the fft size
	cv::Mat uncropFFT( cv::Mat const & input, cv::Size const & oldSize );

  //! Resizes the input image in place
  /*! @param input The image
      @param sz The new size, or an empty size to use the scaling factors
      @param dx The scaling factor along the x dimension, assuming sz is empty
      @param dy The scaling factor along the y dimension, assuming sz is empty */
	void resizeInPlace( cv::Mat & input, cv::Size const & sz = cv::Size(), double dx = 0, double dy = 0 );
}

#endif // UTIL_H_
