#include "../src/Generate.H"
#include "Util.H"
#include "BorderOwnershipTests.H"
#include "../config/Options.H"

#include <fstream>

extern OptionsStruct Options;

void testSquareOverlap( bd::Border & border )
{
  const int rS = border.itsResolutionScaling;
  const int height = (int)border.itsGridHeight * rS;
  const int width = (int)border.itsGridWidth * rS;
  const int squareSize = 10 * rS;

  cv::Mat mask = cv::Mat::zeros( height, width, CV_32FC1 );

  auto s1 = bd::generateRectangle( 15 * rS, 15 * rS, 10 * rS, 10 * rS );
  auto s2 = bd::generateRectangle( 20 * rS, 20 * rS, 10 * rS, 10 * rS );
  //auto s2 = bd::generateRectangle( 24, 24, 10, 10 );

  cv::Mat stimulus = cv::Mat::zeros( height, width, CV_32FC1 );
  //bd::drawShape( stimulus, s1, 1.0, false );
  //bd::drawShape( stimulus, s2, 0.0, true );
  //bd::drawShape( stimulus, s2, 1.0, false );
  bd::drawShapesInterpolated( stimulus, {{s1, 1.0, false, rS},
                                         {s2, 0.0, true, rS},
                                         {s2, 1.0, false, rS}} );
  //bd::drawShape( stimulus, bd::generateRectangle( 25, 15, 15, 15 ), 0.0, true );
  //bd::drawShape( stimulus, bd::generateRectangle( 15, 25, 15, 15 ), 0.0, true );

  bd::showImage( stimulus, false );

  const int numPresentIters = 10; // how long to show each square at each location

  displayPolarity(border.itsBOResponses[0], border.itsGNResponses, false) ;
  size_t iter = 0;
  while( true )
  {
    bd::test::presentStimulus( border, stimulus, 1 );

    showConnections( border.itsGNResponses, false );
    showConnections( border.itsBOResponses[0], border.itsGNResponses, false );
    bd::showImage( visualizeInhibitoryPairs( border.itsBOResponses[0] ), false );
    bd::showImage( visualizePrimaryDirection( border.itsBOResponses[0] ), false );

    const std::string saveName = "square_overlap_iter_" + std::to_string(iter++);
    bd::test::dumpBO( border, saveName );
  }
}

void testSquareCross( bd::Border & border )
{
  const int rS = border.itsResolutionScaling;
  const int height = (int)border.itsGridHeight * rS;
  const int width = (int)border.itsGridWidth * rS;

  cv::Mat mask = cv::Mat::zeros( height, width, CV_32FC1 );

  auto makeRoundedSquare = [=](std::vector<bd::Shape> & shapes, int x, int y)
  {
    const int size = 8 * rS; // total width of square
    const int c_size = 1.5 * rS; // cut-off for each rounded corner

    // start with top horizontal, move clockwise doing flat faces
    shapes.emplace_back( bd::generateLine( x + c_size, y,          x + size - c_size, y ) );
    shapes.emplace_back( bd::generateLine( x + size,   y + c_size, x + size,          y + size - c_size ) );
    shapes.emplace_back( bd::generateLine( x + c_size, y + size,   x + size - c_size, y + size ) );
    shapes.emplace_back( bd::generateLine( x,          y + c_size, x,                 y + size - c_size ) );

    // start with upper left, move clockise on diagonal pieces
    shapes.emplace_back( bd::generateLine( x,                 y + c_size,        x + c_size,        y ) );
    shapes.emplace_back( bd::generateLine( x + size - c_size, y,                 x + size,          y + c_size ) );
    shapes.emplace_back( bd::generateLine( x + size,          y + size - c_size, x + size - c_size, y + size ) );
    shapes.emplace_back( bd::generateLine( x + c_size,        y + size,          x,                 y + size - c_size ) );
  };


  //   [] s1
  // []  [] s2, 23
  //   [] s4
//  auto s1 = bd::generateRectangle( 22 * rS, 8 * rS, 8 * rS, 8 * rS );
//  auto s2 = bd::generateRectangle( 14 * rS, 16 * rS, 8 * rS, 8 * rS );
//  auto s3 = bd::generateRectangle( 30 * rS, 16 * rS, 8 * rS, 8 * rS );
//  auto s4 = bd::generateRectangle( 22 * rS, 24 * rS, 8 * rS, 8 * rS );

  std::vector<bd::Shape> shapes;
  makeRoundedSquare( shapes, 22 * rS, 8  * rS );
  makeRoundedSquare( shapes, 14 * rS, 16 * rS );
  makeRoundedSquare( shapes, 30 * rS, 16 * rS );
  makeRoundedSquare( shapes, 22 * rS, 24 * rS );

  cv::Mat stimulus = cv::Mat::zeros( height, width, CV_32FC1 );

  std::vector<bd::ShapeMetaData> shapeMetaData( shapes.size() );
  for( int i = 0; i < shapeMetaData.size(); ++i )
    shapeMetaData[i] = {shapes[i], 1.0, false, rS};

  bd::drawShapesInterpolated( stimulus, shapeMetaData );
//  bd::drawShapesInterpolated( stimulus, {{s1, 1.0, false, rS},
//                                         {s2, 1.0, false, rS},
//                                         {s3, 1.0, false, rS},
//                                         {s4, 1.0, false, rS}} );
  bd::showImage( stimulus, false );

  const int numPresentIters = 10; // how long to show each square at each location

  //displayPolarity(border.itsBOResponses[0], border.itsGNResponses, false) ;
  size_t iter = 0;
  while( true )
  {
    bd::test::presentStimulus( border, stimulus, 1 );

    showConnections( border.itsGNResponses, false );
    showConnections( border.itsBOResponses[0], border.itsGNResponses, false );
    bd::showImage( visualizeInhibitoryPairs( border.itsBOResponses[0] ), false );
    bd::showImage( visualizePrimaryDirection( border.itsBOResponses[0] ), false );

    const std::string saveName = "square_cross_iter_" + std::to_string(iter++);
    bd::test::dumpBO( border, saveName );
  }
}

void testTetrisPiece1( bd::Border & border )
{
  const auto oldState = border.itsState;
  border.itsState.learningRate = 0.0;

  const int rS = border.itsResolutionScaling;
  const int height = (int)border.itsGridHeight * rS;
  const int width = (int)border.itsGridWidth * rS;
  const int squareSize = 9 * rS;

  cv::Mat mask = cv::Mat::zeros( height, width, CV_32FC1 );

/* []  []
 * [][][] */
  auto s1 = bd::generateRectangle( 8 * rS, 17 * rS, 34 * rS, 8 * rS ); // bottom rectangle
  auto s2 = bd::generateRectangle( 8 * rS,  9 * rS, 8 * rS,  8 * rS ); // upper left square
  auto s3 = bd::generateRectangle( 34 * rS, 9 * rS, 8 * rS,  8 * rS ); // upper right square

  auto s4 = bd::generateRectangle( 8.6 * rS,  15 * rS, 6.8 * rS, 3 * rS ); // left side blackout
  auto s5 = bd::generateRectangle( 34.6 * rS, 15 * rS, 6.8 * rS, 3 * rS ); // right side blackout

  cv::Mat stimulus = cv::Mat::zeros( height, width, CV_32FC1 );
//  bd::drawShape( stimulus, s1, 1.0, false );
//  bd::drawShape( stimulus, s2, 1.0, false );
//  bd::drawShape( stimulus, s3, 1.0, false );
//  bd::drawShape( stimulus, bd::generateLine( 6, 19, 7 ), 0.0 );
//  bd::drawShape( stimulus, bd::generateLine( 26, 19, 7 ), 0.0 );

  bd::drawShapesInterpolated( stimulus, {{s1, 1.0, false, rS},
                                         {s2, 1.0, false, rS},
                                         {s3, 1.0, false, rS},
                                         {s4, 0.0, true, rS},
                                         {s5, 0.0, true, rS}});

  bd::showImage( stimulus , false );

  //displayPolarity(border.itsBOResponses[0], border.itsGNResponses, false) ;
  size_t iter = 0;
  while( true )
  {
    bd::test::presentStimulus( border, stimulus, 1 );

    showConnections( border.itsGNResponses, false );
    showConnections( border.itsBOResponses[2], border.itsGNResponses, false );
    bd::showImage( visualizeInhibitoryPairs( border.itsBOResponses[2] ), false );
    bd::showImage( visualizePrimaryDirection( border.itsBOResponses[2] ), false );
    bd::showImage( visualizePrimaryDirection( border.itsBOResponses[0] ), false );

    const std::string saveName = "tetris_iter_" + std::to_string(iter++);
    bd::test::dumpBO( border, saveName );
  }

  border.itsState = oldState;
}

template <size_t N>
struct Generator
{
  static_assert(N < 64, "Must fit in uint64_t");

  class Configuration
  {
    public:
      using block_type = std::array<std::array<uint8_t, N>, N>;
      block_type config;

      Configuration() : config{}
      { }

      //! Construct from a number
      Configuration( uint64_t number )
      {
        auto * b = config[0].data();
        constexpr int size = N*N;
        for( int i = 0; i < size; ++i )
          b[i] = (number & (1 << i)) ? 1 : 0;
      }

      //! Rotates this configuration counter-clockwise by 90 degrees
      Configuration rotate() const
      {
        Configuration rotated;

        for( size_t y = 0; y < N; ++y )
          for( size_t x = 0; x < N; ++x )
            rotated.config[N-x-1][y] = config[y][x];

        return rotated;
      }

      //! Flips this configuration along a horizontal axis
      Configuration flipUD() const
      {
        Configuration flipped;

        for( size_t y = 0; y < N; ++y )
          for( size_t x = 0; x < N; ++x )
            flipped.config[y][x] = config[N-y-1][x];

        return flipped;
      }

      //! Translates by the requested amount if valid
      /*! Invalid translations (out of bounds) will return false for second
          parameter and an uninitialized Configuration */
      std::pair<Configuration, bool> translate( int yy, int xx ) const
      {
        Configuration translated{};

        const int iN = N;

        for( int y = 0; y < iN; ++y )
          for( int x = 0; x < iN; ++x )
          {
            const bool inBounds = (x + xx < iN)  &&
                                  (x + xx >= 0 ) &&
                                  (y + yy < iN)  &&
                                  (y + yy >= 0);

            if( inBounds )
              translated.config[y+yy][x+xx] = config[y][x];
            else if( config[y][x] )
              return {{}, false};
          }

        return {translated, true};
      }

      void print() const
      {
        for( size_t y = 0; y < N; ++y )
        {
          for( size_t x = 0; x < N; ++x )
            if( config[y][x] )
              std::cout << "X";
            else
              std::cout << "_";
          std::cout << std::endl;
        }
      }

      //! Returns as a number
      uint64_t hash() const
      {
        uint64_t asNumber = 0;

        auto * b = config[0].data();
        constexpr int size = N*N;
        for( int i = 0; i < size; ++i, ++b )
        {
          const uint64_t bit = *b ? 1 : 0;
          asNumber = asNumber | (bit << i );
        }

        return asNumber;
      }

      bool operator<( Configuration const & other ) const
      {
        return hash() < other.hash();
      }

      //! Returns the "scale" of this shape
      /*! Shapes get a scale based on the largest square that can fit
          inside them. A 1x1 square gets a scale of 1. */
      size_t scale() const
      {
        // Check largest scale then recursively check locations for smaller scales.
        // We don't need to check smallest scale since it is true by construction
        auto getSum = [&](size_t xx, size_t yy, size_t scale)
        {
          int sum = 0;
          for( size_t y = 0; y < scale && y + yy < N; ++y )
            for( size_t x = 0; x < scale && x + xx < N; ++x )
              sum += config[y + yy][x + xx] ? 1 : 0;

          return sum;
        };

        for( size_t currentScale = N; currentScale > 1; --currentScale )
        {
          for( size_t y = 0; y < N && ((y + currentScale) <= N); ++y )
            for( size_t x = 0; x < N && ((x + currentScale) <= N); ++x )
              if( getSum( x, y, currentScale ) == (currentScale * currentScale) )
                return currentScale;
        }

        return 1;
      }

      //! Returns the "bounding scale" of this shape
      /*! This is the smallest square that can contain this shape entirely */
      size_t boundingScale() const
      {
        size_t xL = N, xR = 0, yT = N, yB = 0;

        for( size_t y = 0; y < N; ++y )
          for( size_t x = 0; x < N; ++x )
          {
            if( !config[y][x] )
              continue;

            xL = std::min( xL, x );
            xR = std::max( xR, x );
            yT = std::min( yT, y );
            yB = std::max( yB, y );
          }

        return std::max( xR - xL, yB - yT ) + 1;
      }

      //! Returns this configuration as a shape
      /*! @param xx Origin x
          @param yy Origin y
          @param squareSize The length of a square */
      std::vector<bd::Shape> toShapes( double xx, double yy, int squareSize, bool fill = false ) const
      {
        const int size = N;

        struct Square
        {
          Square() : valid( false ) {}
          Square( bool u, bool d, bool l, bool r ) :
            valid( !(u & d & l & r) ),
            up(u), down(d), left(l), right(r) {}
          bool valid;
          bool up, down, left, right;
        };

        std::array<std::array<Square, N>, N> squares;

        auto hasNeighbor = [&](int y, int x)
        {
          if( x < 0 || x >= size )
            return false;
          if( y < 0 || y >= size )
            return false;

          if( config[y][x] )
            return true;
          else
            return false;
        };

        // Assemble valid squares and mark which sides have neighbors.
        // A square is invalid if it has neighbors on all sides unless fill
        // is set, in which case all squares will be painted
        for( size_t y = 0; y < N; ++y )
          for( size_t x = 0; x < N; ++x )
          {
            if( config[y][x] )
            {
              squares[y][x] = { hasNeighbor(y-1, x),
                                hasNeighbor(y+1, x),
                                hasNeighbor(y, x-1),
                                hasNeighbor(y, x+1) };
              if( fill )
                squares[y][x].valid = true;
            }
          }

        // Ready to construct shapes from valid squares
        // Since we are building a vector of shapes, the centroid for each will be the
        // centroid of the whole.
        std::vector<bd::Shape> shapes;

        auto makeSquare = [&]( size_t y, size_t x, Square const & s )
        {
          // Buffer size because openCV anti-aliasing results in gaps when we do rotations
          const int buffer = 1;
          const int topLeftX = xx + x * squareSize - buffer;
          const int topLeftY = yy + y * squareSize - buffer;

          return bd::generateRectangle( topLeftX, topLeftY, squareSize + 2*buffer, squareSize + 2*buffer );
        };

        auto makeLines = [&]( size_t y, size_t x, Square const & s )
        {
          const int topLeftX = xx + x * squareSize;
          const int topLeftY = yy + y * squareSize;

          const int bottomLeftX = topLeftX;
          const int bottomLeftY = topLeftY + squareSize - 1;

          const int topRightX = topLeftX + squareSize - 1;
          const int topRightY = topLeftY;

          const int bottomRightX = topLeftX + squareSize - 1;
          const int bottomRightY = topLeftY + squareSize - 1;

          std::vector<bd::Shape> lines;

          // Always create lines using clockwise elements
          if( !s.up )
            lines.emplace_back( bd::generateLine( topLeftX, topLeftY, topRightX, topRightY, true ) );
          if( !s.right )
            lines.emplace_back( bd::generateLine( topRightX, topRightY, bottomRightX, bottomRightY, true ) );
          if( !s.down )
            lines.emplace_back( bd::generateLine( bottomRightX, bottomRightY, bottomLeftX, bottomLeftY, true ) );
          if( !s.left )
            lines.emplace_back( bd::generateLine( bottomLeftX, bottomLeftY, topLeftX, topLeftY, true ) );

          return lines;
        };

        double centerX = 0, centerY = 0;
        size_t count = 0;

        for( size_t y = 0; y < N; ++y )
          for( size_t x = 0; x < N; ++x )
          {
            if( squares[y][x].valid )
            {
              if( fill )
                shapes.emplace_back( makeSquare( y, x, squares[y][x] ) );
              else
              {
                auto lines = makeLines( y, x, squares[y][x] );
                shapes.insert( shapes.end(), lines.begin(), lines.end() );
              }

              centerX += x * squareSize + squareSize / 2.0 - 0.5;
              centerY += y * squareSize + squareSize / 2.0 - 0.5;
              ++count;
            }
          } // end for

        centerX = (centerX / count) + xx;
        centerY = (centerY / count) + yy;

        for( auto & shape : shapes )
          shape.centroid = {centerX, centerY};

        return shapes;
      }

      //! Checks if a configuration is valid
      /*! A configuration is valid if it satisfies two contraints:
          1) Only a single connected component
          2) No holes */
      bool isValid()
      {
        // Step 1: check for one CC in ON pixels
        const bool onValid = hasOneConnectedComponent( config );
        if( !onValid )
          return false;

        // Step 2: check that there are no holes by inverting and seeing if all reachable from outside
        static const int size = N+2;
        std::array<std::array<uint8_t, N+2>, N+2> inverted;
        for( int y = 0; y < size; ++y )
        {
          for( int x = 0; x < size; ++x )
          {
            if( y > 0 && y < size-1 && x > 0 && x < size-1 )
              inverted[y][x] = config[y-1][x-1] ? 0 : 1;
            else
              inverted[y][x] = 1;
          }
        }

        const bool invertedValid = hasOneConnectedComponent( inverted );
        return invertedValid;
      }

    private:

      enum Labels
      {
        OFF = 0,
        ON,
        MARKED
      };

      template <size_t M> inline
      bool hasOneConnectedComponent( std::array<std::array<uint8_t, M>, M> blocks )
      {
        const static int size = M;
        std::stack<std::pair<int, int>> next;

        auto mark = [&](int y, int x)
        {
          if( x < 0 || x >= size )
            return;
          if( y < 0 || y >= size )
            return;

          if( blocks[y][x] && blocks[y][x] != Labels::MARKED )
          {
            blocks[y][x] = Labels::MARKED;
            next.emplace(y,x);
          }
        };

        // Get starting location
        for( int y = 0; y < size; ++y )
          for( int x = 0; x < size; ++x )
            if( blocks[y][x] && next.empty() )
            {
              next.emplace(y,x);
              blocks[y][x] = Labels::MARKED;
            }

        if( next.empty() ) // no active locations
          return false;

        // Find all connected components
        while( !next.empty() )
        {
          int y, x;
          std::tie(y, x) = next.top(); next.pop();
          {
            mark(y, x-1);
            mark(y, x+1);
            mark(y-1, x);
            mark(y+1, x);
          }
        }

        // Invalid if any on values are not in CC
        for( int y = 0; y < size; ++y )
          for( int x = 0; x < size; ++x )
            if( blocks[y][x] && blocks[y][x] != Labels::MARKED )
              return false;

        return true;
      }
  };

  //! Generates all valid configurations of blocks and
  //! partitions them into a training and test set
  Generator( double percentageTrain, bool randomize, size_t seed )
  {
    std::uint64_t gen = 0x0;
    const uint64_t max = std::pow(2, N*N);

    while( true )
    {
      Configuration config(gen++);

      if( gen == 0 || gen > max ) // done with everything
        break;

      auto checkRotations = [&]( Configuration const & c )
      {
        auto rot90  = c.rotate();
        auto rot180 = rot90.rotate();
        auto rot270 = rot180.rotate();

        if( validConfigurations.count( c )      ||
            validConfigurations.count( rot90 )  ||
            validConfigurations.count( rot180 ) ||
            validConfigurations.count( rot270 ) )
          return false;
        else
          return true;
      };

      if( config.isValid() )
      {
        bool allValid = true;

        for( int y = -(int)N; y < (int)N; ++y )
        {
          for( int x = -(int)N; x < (int)N; ++x )
          {
            auto translated = config.translate( y, x );
            if( translated.second )
              allValid &= checkRotations(translated.first);

            if( !allValid )
              goto configTransformsDone;
          }
        }

        configTransformsDone:

        if( allValid )
          validConfigurations.insert( config );
      }
    }

    // Partition the sets
    std::vector<Configuration> allConfigs;
    for( auto const & c : validConfigurations )
      allConfigs.push_back( c );

    if( randomize )
    {
      std::mt19937 rng(seed);
      std::shuffle( allConfigs.begin(), allConfigs.end(), rng );
    }

    const size_t splitPoint = allConfigs.size() * percentageTrain;
    trainSet.insert( trainSet.begin(), allConfigs.begin(), allConfigs.begin() + splitPoint );
    testSet.insert( testSet.begin(), allConfigs.begin() + splitPoint, allConfigs.end() );
  }

  std::set<Configuration> validConfigurations;
  std::vector<Configuration> trainSet;
  std::vector<Configuration> testSet;
};

void test4x4Configurations( bd::Border & border, const double groupingScale, const TestConfiguration tc, const size_t seed )
{
  const bool test = tc == TestConfiguration::Test || tc == TestConfiguration::Test_output;
  const bool train = !test;

  const int rS = border.itsResolutionScaling;
  const int height = (int)border.itsGridHeight * rS;
  const int width = (int)border.itsGridWidth * rS;

  const int maxSteps = (int)std::max( border.itsGridHeight, border.itsGridWidth );
  const int maxScaleSteps = maxSteps / 6; // maximum time to change scale
  const cv::Mat mask = cv::Mat::zeros( height, width, CV_32FC1 );

  // Parameters
  const bool fill = false;
  const double staticMinScale = 1.0;
  const double staticMaxScale = 1.0;
  const double dynamicMinScale = 1.0; // .75/2.5
  const double dynamicMaxScale = 1.0;
  const int squareSize = 10 * groupingScale * rS;

  // Presentation parameters
  const int numPresentIters = 13; // how long to show each square at each location
  const int numOverallIters = 40001;

  static constexpr size_t GENSIZE = 4;
  Generator<GENSIZE> generator(train ? 1.0 : 0.0, train, seed);
  const auto & shapeSet = train ? generator.trainSet : generator.testSet;

  // Rotates, translates, and/or scales a set of shapes
  auto transformShape = []( std::vector<bd::ShapeMetaData> & shapes, double x, double y, double rad, double scale )
  {
    std::for_each( shapes.begin(), shapes.end(), [=](bd::ShapeMetaData & s)
        {
          s.shape = bd::scaleShape( bd::rotateShape( bd::translateShape( s.shape, {x, y} ), rad ), scale );
        } );
  };

  auto getNextShape = [&](size_t index, double & scale)
  {
    const auto shapes = shapeSet[index].toShapes( 0, 0, squareSize, fill );

    std::vector<bd::ShapeMetaData> shapeMetaData( shapes.size() );
    for( size_t i = 0; i < shapeMetaData.size(); ++i )
      shapeMetaData[i] = {shapes[i], 1.0, fill, rS};

    const int centerX = shapeMetaData[0].shape.centroid.x;
    const int centerY = shapeMetaData[0].shape.centroid.y;

    if( test ) // position in center, no rotation or scaling
    {
      std::cout << centerX << " " << centerY << std::endl;
      transformShape( shapeMetaData, width / 2 - centerX, height / 2 - centerY,
                                     0.0, 1.0 );
      std::cout << shapeMetaData[0].shape.centroid.x << " " << shapeMetaData[0].shape.centroid.y << std::endl;

    }
    else // train
    {
      const double theta = bd::uniformReal( 0.0, 2 * M_PI )();

      transformShape( shapeMetaData, bd::uniformInt( -centerX - 1, width - centerX )(),
                                     bd::uniformInt( -centerY - 1, height - centerY )(),
                                     theta, scale );
    }

    return shapeMetaData;
  };

  // choose a random direction towards any other quadrant
  // NOTES: this tends to result in a strong center bias
  auto otherQuadrantDxDy = [](double width, double height, cv::Point2d centroid)
  {
    const double x = centroid.x;
    const double y = centroid.y;

    double minRNG = 0.0;
    double maxRNG = 2 * M_PI;

    if( x < width / 2 )
    {
      if( y < height / 2 ) // upper left
      {
        minRNG = 3.0 / 2.0 * M_PI;
        maxRNG = 2.0 * M_PI;
      }
      else // bottom left
      {
        minRNG = 0.0;
        maxRNG = M_PI / 2.0;
      }
    }
    else
    {
      if( y < height / 2 ) // upper right
      {
        minRNG = M_PI;
        maxRNG = 3.0 / 2.0 * M_PI;
      }
      else // bottom right
      {
        minRNG = M_PI / 2.0;
        maxRNG = M_PI;
      }
    }

    const double theta = bd::uniformReal( minRNG, maxRNG )();
    return cv::Point2d( std::cos(theta), -std::sin(theta) );
  };

  // Chooses a random end point and sets the dx-dy step-size to move towards it
  auto randomEndpointDxDy = [maxSteps, squareSize](int width, int height, cv::Point2d centroid)
  {
    const cv::Point2d endPoint( bd::uniformInt( -squareSize - 1, width - 1 )(),
                                bd::uniformInt( -squareSize - 1, height - 1)() );

    auto dxdy = endPoint - centroid;
    dxdy.x /= maxSteps; dxdy.y /= maxSteps;
    return dxdy;
  };

  // choose a random direction in radians, 0 to 2pi, with no regard for maximum steps
  auto randomDxDy = []()
  {
    const double theta = bd::uniformReal( 0.0, 2 * M_PI )();
    return cv::Point2d( std::cos(theta), -std::sin(theta) );
  };

  auto anyIsValid = [&]( std::vector<bd::ShapeMetaData> const & shapes )
  {
    return std::any_of( shapes.begin(), shapes.end(), [&](bd::ShapeMetaData const & s)
        { return bd::isValid( border, s.shape, rS ); } );
  };

  size_t shapeCounter = 0;

  // Two types of movements:
  //  1) Random direction with subtle zoom
  //  2) Stay in place and zoom
  for( int i = 0; i < numOverallIters; ++i )
  {
    // If a shape has a base scale greater than 1, we won't end up scaling or being static
    const int shapeScale = shapeSet[shapeCounter].scale();
    const int boundingScale = shapeSet[shapeCounter].boundingScale();
    const bool baseScaled = shapeScale > 1;

    //const bool isMoving = baseScaled ? true : bd::uniformInt( 0, 4 )(); // 1 in 5 is static
    const bool isMoving = true;
    const double minScale = baseScaled ? 1.0 : isMoving ? dynamicMinScale : staticMinScale;
    const double maxScale = baseScaled ? 1.0 : isMoving ? dynamicMaxScale : staticMaxScale;

    double currentScale = bd::uniformReal( minScale, maxScale )();

    auto shapes = getNextShape( shapeCounter, currentScale );

    //double scaleStep = std::exp( std::log( bd::uniformReal( minScale, maxScale )() ) / maxScaleSteps );
    double scaleStep = 1.0; // scale static once chosen
    //auto dxdy = otherQuadrantDxDy(width, height, shapes[0].shape.centroid) * rS;
    //auto dxdy = randomEndpointDxDy(width, height, shapes[0].shape.centroid);
    auto dxdy = isMoving ? randomDxDy() * rS : cv::Point2d{0.0, 0.0};

    const auto scoresPrefix = string_sprintf( "%dx%d_%d_gen_overlap_shape_%04d", GENSIZE, GENSIZE, boundingScale, shapeCounter );
    std::ofstream scoresFile( scoresPrefix + "_scores.txt" );
    if( tc != TestConfiguration::Train )
    {
      std::cout << "Shape id: " << shapeCounter << std::endl;
      shapeSet[shapeCounter].print();
    }

    bool displayShape = true;
    int step = 0;
    while( anyIsValid( shapes ) && step < maxSteps )
    {
      cv::Mat stimulus = cv::Mat::zeros( height, width, CV_32FC1 );
      bd::drawShapesInterpolated( stimulus, shapes );

      bool finishDisplay = true;
      for( int z = 0; z < numPresentIters; ++z )
      {
        bd::test::presentStimulus( border, stimulus, 1 );

        if( finishDisplay && displayShape )
        {
          cv::Mat gtCanvas = cv::Mat::zeros( height, width, CV_32FC1 );
          cv::Mat gt   = bd::drawGroundTruth( gtCanvas, shapes );
          cv::Mat dirs = bd::visualizeAllDirections( border.itsBOResponses, rS );

          if( Options.display && tc != TestConfiguration::Test_output ) // Don't display anything for test output
          {
            bd::showImage(stimulus, false, false, "input");
            bd::showImage(gt, false, false, "color");
            bd::showImage( dirs, false, false, "bo" );

            showConnections( border.itsGNResponses, false, false, "grouping" );
            showConnections( border.itsProtoResponses, false, false, "proto" );

            showConnections( border.itsBOResponses[0], border.itsGNResponses, false, "bo_0_0" );
            showConnections( border.itsBOResponses[1], border.itsGNResponses, false, "bo_45_0" );
            showConnections( border.itsBOResponses[2], border.itsGNResponses, false, "bo_90_0" );
            showConnections( border.itsBOResponses[3], border.itsGNResponses, false, "bo_135_0" );

            //displayPolarity( border.itsBOResponses[0], border.itsGNResponses, false, "polarity_0_0" );
            //displayPolarity( border.itsBOResponses[1], border.itsGNResponses, false, "polarity_45_0" );
            //displayPolarity( border.itsBOResponses[2], border.itsGNResponses, false, "polarity_90_0" );
            //displayPolarity( border.itsBOResponses[3], border.itsGNResponses, false, "polarity_135_0" );
          }

          auto polarityScores = bd::test::activePolarityScore( gt, dirs, rS );

          if( Options.display && tc != TestConfiguration::Test_output )
          {
            char key = cv::waitKey(0);
            if( key == 'n' ) // skip to next position of shape
              finishDisplay = false;
            else if (key == 'k' ) // skip to next shape, with iteration
              displayShape = false;
            else if (key == 'q' )  // skip to next shape, with no iteration
            {
              step = maxSteps;
              break;
            }
          }
          else if( z == (numPresentIters - 1)) // testoutput
          {
            auto polarityString = string_sprintf( "%f, %f, %f, %f, %f, %f",    std::get<0>(polarityScores),
                                                  std::get<1>(polarityScores), std::get<2>(polarityScores),
                                                  std::get<3>(polarityScores), std::get<4>(polarityScores),
                                                  std::get<5>(polarityScores) );

            const auto savePrefix = string_sprintf( "%dx%d_%d_gen_overlap_shape_%04d_counter_%03d", GENSIZE, GENSIZE, boundingScale, shapeCounter, z );
            bd::test::dumpBO( border, savePrefix );

            scoresFile << polarityString << std::endl;
          }
        } // end display
      } // end present loop

      if( test ) // only present at a single location
        break;

      currentScale *= scaleStep;
      if( currentScale < minScale || currentScale > maxScale ) // scale is over limit
      {
        scaleStep = 1.0 / scaleStep; // reverse scaling direction
        currentScale *= scaleStep;
      }

      transformShape( shapes, dxdy.x, dxdy.y, 0.0, scaleStep );

      ++step;
    } // end while over current shape

    ++shapeCounter;
    if( shapeCounter >= shapeSet.size() )
    {
      shapeCounter = 0;

      if( tc == TestConfiguration::Test_output )
        break; // break main loop
    }

    bd::test::doMask( border, mask );

    if( train )
    {
      const int numInspectIters = Options.iters;
      if( i % numInspectIters == 0 && i != 0 )
      {
        bd::test::saveBD( border, Options.output + "_" + std::to_string(i) + "_4x4_converge.cereal" );

        std::cout << "=================================" << std::endl;
        std::cout << "=================================" << std::endl;
        std::cout << "Done with " << i << " iterations " << std::endl;
        std::cout << "=================================" << std::endl;
        std::cout << "=================================" << std::endl;
      }
      else
        std::cout << "Counter: " << (i % numInspectIters) << std::endl;
    }
  }
}

void test4x4SpecificConfiguration( bd::Border & border, const double groupingScale,
                                   const size_t shapeIndex,
                                   const TestConfiguration tc, const SpecificTest st,
                                   const size_t seed )
{
  const int rS = border.itsResolutionScaling;
  const int height = (int)border.itsGridHeight * rS;
  const int width = (int)border.itsGridWidth * rS;

  const int maxSteps = (int)std::max( border.itsGridHeight, border.itsGridWidth );
  const cv::Mat mask = cv::Mat::zeros( height, width, CV_32FC1 );

  // Parameters
  const bool fill = false;
  const int squareSize = 10 * groupingScale * rS;

  const int maxScaleSteps = maxSteps / 6; // maximum time to change scale
  const double minScale = st == SpecificTest::Scale ? 0.5 : 1.0;
  const double maxScale = st == SpecificTest::Scale ? 2.6 : 1.0; // 4.1 for square, 2.6 for c shape

  const int rotationSteps = 16;
  const double radianStep = 2.0 * M_PI / static_cast<double>(rotationSteps);

  const int translationSteps = 10;
  const double xStep = width / static_cast<double>(translationSteps);
  const double yStep = height / static_cast<double>(translationSteps);

  // Presentation parameters
  const int numPresentIters = 13; // how long to show each square at each location

  static constexpr size_t GENSIZE = 4;
  Generator<GENSIZE> generator(0.0, false, seed);
  const auto & shapeSet = generator.testSet;

  // Rotates, translates, and/or scales a set of shapes
  auto transformShape = []( std::vector<bd::ShapeMetaData> & shapes, double x, double y, double rad, double scale )
  {
    std::for_each( shapes.begin(), shapes.end(), [=](bd::ShapeMetaData & s)
        {
          s.shape = bd::scaleShape( bd::rotateShape( bd::translateShape( s.shape, {x, y} ), rad ), scale );
        } );
  };

  auto getNextShape = [&](size_t index, double & scale)
  {
    const auto shapes = shapeSet[index].toShapes( 0, 0, squareSize, fill );

    std::vector<bd::ShapeMetaData> shapeMetaData( shapes.size() );
    for( size_t i = 0; i < shapeMetaData.size(); ++i )
      shapeMetaData[i] = {shapes[i], 1.0, fill, rS};

    const int centerX = shapeMetaData[0].shape.centroid.x;
    const int centerY = shapeMetaData[0].shape.centroid.y;

    if( st == SpecificTest::Translation ) // place in upper left corner
      transformShape( shapeMetaData, -centerX, -centerY, 0.0, scale );
    else // place in center
      transformShape( shapeMetaData, width / 2 - centerX, height / 2 - centerY,
                                     0.0, scale );

    return shapeMetaData;
  };

  auto anyIsValid = [&]( std::vector<bd::ShapeMetaData> const & shapes )
  {
    return std::any_of( shapes.begin(), shapes.end(), [&](bd::ShapeMetaData const & s)
        { return bd::isValid( border, s.shape, rS ); } );
  };

  // Present the shape
  {
    shapeSet[shapeIndex].print();
    const int shapeScale = shapeSet[shapeIndex].scale();
    const int boundingScale = shapeSet[shapeIndex].boundingScale();

    double currentScale = minScale;

    auto shapes = getNextShape( shapeIndex, currentScale );

    //double scaleStep = st == SpecificTest::Scale ? std::exp( std::log( maxScale ) / maxScaleSteps ) : 1.0;
    double scaleStep = st == SpecificTest::Scale ? 0.1 : 0.0;

    const std::string testName = st == SpecificTest::Static      ? "static"   :
                                 st == SpecificTest::Scale       ? "scale"    :
                                 st == SpecificTest::Rotation    ? "rotation" :
                              /* st == SpecificTest::Translation ?*/ "translation";

    int xPos = 0; // used for translation
    int yPos = 0;

    bool displayShape = true;
    int step = 0;
    while( anyIsValid( shapes ) )
    {
      cv::Mat stimulus = cv::Mat::zeros( height, width, CV_32FC1 );
      bd::drawShapesInterpolated( stimulus, shapes );

      const auto scoresPrefix = string_sprintf( "%dx%d_%d_specific_test_%s_shape_%04d_position_%03d", GENSIZE, GENSIZE, boundingScale, testName.c_str(), shapeIndex, step );
      std::ofstream scoresFile( scoresPrefix + "_scores.txt" );

      bool finishDisplay = true;
      for( int z = 0; z < numPresentIters; ++z )
      {
        bd::test::presentStimulus( border, stimulus, 1 );

        if( finishDisplay && displayShape )
        {
          auto stimulusImage = stimulus.clone(); // for saving
          cv::Mat gtCanvas = cv::Mat::zeros( height, width, CV_32FC1 );
          cv::Mat gt   = bd::drawGroundTruth( gtCanvas, shapes );
          auto gtImage = gt.clone(); // for saving

          cv::Mat dirs = bd::visualizeAllDirections( border.itsBOResponses, rS );
          auto dirsImage = dirs.clone(); // for saving

          if( Options.display )
          {
            bd::showImage(stimulus, false, false, "input");
            bd::showImage(gt, false, false, "color");

            showConnections( border.itsGNResponses, false, false, "grouping" );
            showConnections( border.itsProtoResponses, false, false, "proto" );

            showConnections( border.itsBOResponses[0], border.itsGNResponses, false, "bo_0_0" );
            showConnections( border.itsBOResponses[1], border.itsGNResponses, false, "bo_45_0" );
            showConnections( border.itsBOResponses[2], border.itsGNResponses, false, "bo_90_0" );
            showConnections( border.itsBOResponses[3], border.itsGNResponses, false, "bo_135_0" );

            //displayPolarity( border.itsBOResponses[0], border.itsGNResponses, false, "polarity_0_0" );
            //displayPolarity( border.itsBOResponses[1], border.itsGNResponses, false, "polarity_45_0" );
            //displayPolarity( border.itsBOResponses[2], border.itsGNResponses, false, "polarity_90_0" );
            //displayPolarity( border.itsBOResponses[3], border.itsGNResponses, false, "polarity_135_0" );

            bd::showImage( dirs, false, false, "bo" );

            if( tc != TestConfiguration::Test_output )
            {
              char key = cv::waitKey(0);
              if( key == 'n' ) // skip to next position of shape
                finishDisplay = false;
              else if (key == 'k' ) // skip to next shape, with iteration
                displayShape = false;
              else if (key == 'q' )  // skip to next shape, with no iteration
              {
                step = maxSteps;
                break;
              }
            }
          } // end Options.display

          // Comment below line for output on every iteration
          if( z == (numPresentIters - 1)) // testoutput
          {
            auto polarityScores = bd::test::activePolarityScore( gt, dirs, rS );
            auto polarityString = string_sprintf( "%f, %f, %f, %f, %f, %f",    std::get<0>(polarityScores),
                                                  std::get<1>(polarityScores), std::get<2>(polarityScores),
                                                  std::get<3>(polarityScores), std::get<4>(polarityScores),
                                                  std::get<5>(polarityScores) );

            const auto savePrefix = string_sprintf( "%dx%d_%d_specific_test_%s_shape_%04d_position_%03d_counter_%03d", GENSIZE, GENSIZE, boundingScale, testName.c_str(), shapeIndex, step, z );
            bd::test::dumpBO( border, savePrefix );

            {
              auto grpImage = sheetActivationImage(border.itsGNResponses);
              auto protoImage = sheetActivationImage(border.itsProtoResponses);

              cv::normalize( grpImage, grpImage, 0, 1, cv::NORM_MINMAX );
              cv::normalize( protoImage, protoImage, 0, 1, cv::NORM_MINMAX );

              cv::imwrite( savePrefix + "_stimulus.png", (1.0 - stimulusImage) * 255 );
              cv::imwrite( savePrefix + "_groundtruth.png", gtImage * 255 );
              cv::imwrite( savePrefix + "_bo.png", dirsImage * 255 );
              cv::imwrite( savePrefix + "_grouping.png", grpImage * 255 );
              cv::imwrite( savePrefix + "_proto.png", protoImage * 255 );

              cv::imwrite( savePrefix + "gabor0.png", border.itsV1Images[0][0] * 255 );
              cv::imwrite( savePrefix + "gabor1.png", border.itsV1Images[1][0] * 255 );
              cv::imwrite( savePrefix + "gabor2.png", border.itsV1Images[2][0] * 255 );
              cv::imwrite( savePrefix + "gabor3.png", border.itsV1Images[3][0] * 255 );
            }

            scoresFile << polarityString << std::endl;
          }
        } // end display
      } // end present loop

      if( st == SpecificTest::Static )
        break; // done after a single presentation
      else if( st == SpecificTest::Scale )
      {
        std::cout << currentScale << std::endl;
        //currentScale *= scaleStep;
        currentScale += scaleStep;
        if( currentScale < minScale || currentScale > maxScale ) // scale is over limit
        {
          // Done with scaling, time to bail!
          break;
        }

        shapes = getNextShape( shapeIndex, currentScale );
        //transformShape( shapes, 0.0, 0.0, 0.0, scaleStep );
      }
      else if( st == SpecificTest::Rotation )
      {
        // done after full rotation set
        if( step > rotationSteps )
          break;

        transformShape( shapes, 0.0, 0.0, radianStep, 1.0 );
      }
      else if( st == SpecificTest::Translation )
      {
        ++xPos;
        transformShape( shapes, xStep, 0.0, 0.0, 1.0 );

        if( xPos > translationSteps )
        {
          xPos = 0;
          ++yPos;
          transformShape( shapes, -(translationSteps+1) * xStep, yStep, 0.0, 1.0 );

          if( yPos > translationSteps )
            break;
        }
      }

      bd::test::doMask( border, mask );

      ++step;
    } // end while over current shape
  } // end present shape
}


namespace
{
  //! Automatically crops and centers a grayscale image
  cv::Mat autocrop( cv::Mat const & image, const int width, const int height )
  {
    int minX = 9999;
    int maxX = -1;
    int minY = 9999;
    int maxY = 0;

    // Get bounding rectangle
    for( int y = 0; y < image.rows; ++y )
      for( int x = 0; x < image.cols; ++x )
        if( image.at<float>( y, x ) > 0 )
        {
          minX = std::min( minX, x );
          maxX = std::max( maxX, x );
          minY = std::min( minY, y );
          maxY = std::max( maxY, y );
        }

    const int cWidth = maxX - minX + 1;
    const int cHeight = maxY - minY + 1;

    std::cout << "Cropping to " << cWidth << " " << cHeight << std::endl;

    cv::Rect boundingBox( minX, minY, cWidth, cHeight );

    // Crop dat
    cv::Mat cropped = image( boundingBox ).clone();

    cv::Mat stimulus;
    int padT = (height - cHeight) / 2;
    int padB = (height - cHeight) / 2;
    int padL = (width - cWidth) / 2;
    int padR = (width - cWidth) / 2;

    if( padT + padB + cHeight < height )
      ++padT;

    if( padL + padR + cWidth < width )
      ++padL;

    cv::copyMakeBorder( cropped, stimulus, padT, padB, padL, padR, cv::BORDER_CONSTANT, 0.0f );

    std::cout << "Stimulus size " << stimulus.cols << " " << stimulus.rows << std::endl;

    cv::dilate( stimulus, stimulus, cv::Mat(), cv::Point(-1,-1), cv::BORDER_CONSTANT, 0.0f );
    cv::dilate( stimulus, stimulus, cv::Mat(), cv::Point(-1,-1), cv::BORDER_CONSTANT, 0.0f );

    return stimulus;
  }
}

void testImage( bd::Border & border, double groupingScale, cv::Mat const & image, TestConfiguration tc )
{
  const int rS = border.itsResolutionScaling;
  const int height = (int)border.itsGridHeight * rS;
  const int width = (int)border.itsGridWidth * rS;

  const cv::Mat mask = cv::Mat::zeros( height, width, CV_32FC1 );
  const int numPresentIters = 13; // how long to show image

  // Present the image
  {
    cv::Mat stimulus = autocrop( image, width, height );

    bool finishDisplay = true;
    for( int z = 0; z < numPresentIters; ++z )
    {
      bd::test::presentStimulus( border, stimulus, 1 );
      cv::Mat dirs = bd::visualizeAllDirections( border.itsBOResponses, rS );

      if( Options.display && finishDisplay )
      {
        bd::showImage(stimulus, false, false, "input");

        showConnections( border.itsGNResponses, false, false, "grouping" );
        showConnections( border.itsProtoResponses, false, false, "proto" );

        showConnections( border.itsBOResponses[0], border.itsGNResponses, false, "bo_0_0" );
        showConnections( border.itsBOResponses[1], border.itsGNResponses, false, "bo_45_0" );
        showConnections( border.itsBOResponses[2], border.itsGNResponses, false, "bo_90_0" );
        showConnections( border.itsBOResponses[3], border.itsGNResponses, false, "bo_135_0" );

        //displayPolarity( border.itsBOResponses[0], border.itsGNResponses, false, "polarity_0_0" );
        //displayPolarity( border.itsBOResponses[1], border.itsGNResponses, false, "polarity_45_0" );
        //displayPolarity( border.itsBOResponses[2], border.itsGNResponses, false, "polarity_90_0" );
        //displayPolarity( border.itsBOResponses[3], border.itsGNResponses, false, "polarity_135_0" );

        bd::showImage( dirs, false, false, "bo" );

        if( tc != TestConfiguration::Test_output )
        {
          char key = cv::waitKey(0);
          if( key == 'n' ) // skip to next position of shape
            finishDisplay = false;
        }
      }

      {
        const auto savePrefix = string_sprintf( "image_test_%03d", z );
        //bd::test::dumpBO( border, savePrefix );

        cv::Mat dirs = bd::visualizeAllDirections( border.itsBOResponses, rS );
        auto dirsImage = dirs.clone(); // for saving
        auto stimulusImage = stimulus.clone(); // for saving

        cv::imwrite( "image_test_stimulus_white.png", (1.0 - stimulusImage) * 255 );
        cv::imwrite( "image_test_stimulus_black.png", (stimulusImage) * 255 );
        cv::imwrite( savePrefix + "_bo.png", dirsImage * 255 );
      }

      std::cout << "Iter " << z << " of " << numPresentIters << std::endl;
    } // end present loop
  } // end present image
}
