#ifndef BORDEROWNERSHIPTESTS_H_
#define BORDEROWNERSHIPTESTS_H_

#include "../src/Border.H"

//! Presents one square on the top corner of a second square, standard size
/*! @note You must load an already learned network! */
void testSquareOverlap( bd::Border & bd );

void testSquareCross( bd::Border & bd );

//! Tests something that looks like:
/*! []  []
 *  [][][] */
void testTetrisPiece1( bd::Border & bd );

//! The configuration to use over a full 4x4 run
enum class TestConfiguration
{
  Train,
  Test,
  Test_output
};


//! Tests all connected configurations of an NxN grid of squares
/*! Squares will be sampled from: [] [] [] []
                                  [] [] [] []
                                  [] [] [] []
                                  [] [] [] []
    with the restriction that each square is 4-connected to its neighbors,
    there is only a single connected component, and there are no holes in
    the selected shape.

    N is hard-coded within the source file.  */
void test4x4Configurations( bd::Border & border, double groupingScale, TestConfiguration tc, size_t seed = std::random_device()() );

//! Used to specify the specific test for 4x4 configurations
enum class SpecificTest
{
  Static,
  Scale,
  Rotation,
  Translation
};

//! Tests a specific shape sampled from an NxN configuration in a specific way
/*! The index of the shape must be specified, as well as the type of test to perform.
    @param border The trained border ownership network
    @param groupingScale The base scale for the grouping layer
    @param shapeIndex The index of the shape to use
    @param tc The test configuration. Train is ignored. Test will display, Test_output will not.
    @param st The specific type of test to run
    @param seed The random seed */
void test4x4SpecificConfiguration( bd::Border & border, double groupingScale,
                                   size_t shapeIndex,
                                   TestConfiguration tc, SpecificTest st,
                                   size_t seed = std::random_device()() );

//! Tests an image by centering it
/*! @param border The trained border ownership network
    @param groupingScale The base scale for the grouping layer
    @param image The contour image to test
    @param tc The test configuration. All ignored except Test_output, which prevents display */
void testImage( bd::Border & border, double groupingScale, cv::Mat const & image, TestConfiguration tc );

#endif // BORDEROWNERSHIPTESTS_H_
