#ifndef CONVERGENCETESTS_H_
#define CONVERGENCETESTS_H_

#include "../src/Border.H"

//! Test a sliding square across an already learned (in opposite direction)
/*! Presents one square at a time with full diagnostics */
void testSlideConvergence( bd::Border & border, cv::Mat const & mask, bool save );

//! Test a single sliding square for convergence
/*! Presents a square at one end of the field and slides it into
    the other side where a second square has already done some learning.

    The end state should be that all border ownership cells along
    this trajectory agree with each other */
void testSimpleConvergence( bd::Border & bd );

//! Slides a square horizontally and then vertically
void testCrossConvergence( bd::Border & bd );

//! Performs a test for full convergence
/*! Presents a square in a random location far enough from the border
    and moves it in some random direction a short distance. */
void testFullConvergence( bd::Border & bd, double scale = 1.0 );

//! Same as full convergence test except movement is done in an orderly spiral from center
void testFullConvergenceSpiral( bd::Border & bd, cv::Mat const & mask );

//! Loads data.cereal and runs a square across a specific row repeatedly
void testConvergenceSpecificRow( bd::Border & bd, cv::Mat const & mask );

//! Performs a test for full convergence using random images (take subwindow over image)
void testFullConvergenceImages( bd::Border & border, std::string const & directory );

#endif // CONVERGENCETESTS_H_
