#include "SimpleTests.H"
#include "../src/Generate.H"
#include "Util.H"

void testConstant( bd::Border & border, cv::Mat const & stimulus,
    int numIters, int skip )
{
  border.onset( stimulus );
  bd::showImage( stimulus, false );
  for( size_t o = 0; o < border.itsBOResponses.size(); ++o )
  {
    bd::showImage( border.itsV1Images[o][0], false );
    showConnections( border.itsBOResponses[o], border.itsGNResponses, false );
  }

  int iter = skip;
  border.iterate(skip);

  while( true )
  {
    border.iterate(numIters);
    iter += numIters;

    std::cout << "=========================" << std::endl;

    showConnections( border.itsGNResponses, false );

    for( const auto & orientation : border.itsBOResponses )
    {
      showConnections( orientation, border.itsGNResponses, false );
      showConnections( border.itsProtoResponses, false, false );
      bd::showImage( visualizeInhibitoryPairs( orientation ), false );
      bd::showImage( visualizePrimaryDirection( orientation ), false );
      bd::showImage( bd::visualizeAllDirections( border.itsBOResponses, 0 ), false, true, "bo_dirs" );

      const std::string saveName = "constant_iter_" + std::to_string(iter);
      bd::test::dumpBO( border, saveName );
    }
  }
}

void testOnOff( bd::Border & border, cv::Mat const & stimulus, cv::Mat const & mask )
{
  border.onset( stimulus );
  border.iterate(100);

  {
    showConnections( border.itsGNResponses, false );
    showConnections( border.itsBOResponses[0], border.itsGNResponses, false );
    bd::showImage( visualizeInhibitoryPairs( border.itsBOResponses[0] ), false );
  }

  border.onset( mask );
  border.iterate(10);

  {
    showConnections( border.itsGNResponses, false );
    showConnections( border.itsBOResponses[0], border.itsGNResponses, false );
    bd::showImage( visualizeInhibitoryPairs( border.itsBOResponses[0] ), false );
  }

  border.onset( stimulus );
  border.iterate(20);

  {
    showConnections( border.itsGNResponses, false );
    showConnections( border.itsBOResponses[0], border.itsGNResponses, false );
    bd::showImage( visualizeInhibitoryPairs( border.itsBOResponses[0] ), false );
  }
}

void testFlip( bd::Border & border, double scale )
{
  const float resolutionScaling = border.itsResolutionScaling;
  const int height = (int)border.itsGridHeight * resolutionScaling;
  const int width = (int)border.itsGridWidth * resolutionScaling;
  const int squareSize = 10 * resolutionScaling * scale;

  const int numIters = 13;

  cv::Mat image1 = cv::Mat::zeros( height, width, CV_32FC1 );
  cv::Mat image2 = cv::Mat::zeros( height, width, CV_32FC1 );
  cv::Mat mask = cv::Mat::zeros( height, width, CV_32FC1 );

  {
    auto shape = bd::generateRectangle( 10 * resolutionScaling * scale, 6 * resolutionScaling * scale, squareSize, squareSize, 1.0 );
    bd::drawShapesInterpolated( image1, {{shape, 255.0, false, (int)resolutionScaling}} );
  }

  {
    auto shape = bd::generateRectangle( 10 * resolutionScaling * scale, 16 * resolutionScaling * scale, squareSize, squareSize, 1.0 );
    bd::drawShapesInterpolated( image2, {{shape, 255.0, false, (int)resolutionScaling}} );
  }

  int counter = 0;
  const int displaySkip = 37;

  while (true)
  {
    border.onset( image1 );
    border.iterate(numIters);

    if( counter % displaySkip == 1 )
    {
      showConnections( border.itsGNResponses, false );
      showConnections( border.itsBOResponses[0], border.itsGNResponses, false );
      bd::showImage( visualizeInhibitoryPairs( border.itsBOResponses[0] ), false );
      displayPolarity(border.itsBOResponses[0], border.itsGNResponses, false) ;
    }

    border.onset( mask );
    border.iterate(numIters);

    border.onset( image2 );
    border.iterate(numIters);

    if( counter % displaySkip == 1 )
    {
      showConnections( border.itsGNResponses, false );
      showConnections( border.itsBOResponses[0], border.itsGNResponses, false );
      bd::showImage( visualizeInhibitoryPairs( border.itsBOResponses[0] ), false );
      displayPolarity(border.itsBOResponses[0], border.itsGNResponses, false) ;
    }

    border.onset( mask );
    border.iterate(numIters);

    ++counter;
  }
}

void testFlipSlide( bd::Border & border, cv::Mat const & mask )
{
  const int height = border.itsGridHeight;
  const int width = border.itsGridWidth;

  auto image = bd::generateRectangle( height, width, 10, 10, 10, 10 );

  auto doMask = [&]()
  {
    border.onset( mask );
    border.iterate( 20 );
  };

  auto presentSquare = [&]( int x, int y, int numIters, std::function<void()> postFunc )
  {
    auto localimage = bd::generateRectangle( height, width, x, y, 10, 10 );

    border.onset( localimage );
    border.iterate(numIters);

    postFunc();
  };

  for( int y = 10; y < 20; ++y )
  {
    //presentSquare( 10, y, 2, [&]()
    //{
    //  showConnections( border.itsGNResponses, false );
    //  bd::showImage( visualizeInhibitoryPairs( border.itsBOResponses[0] ), false );
    //} );

    presentSquare( 10, y, 13, [&]()
    {
      showConnections( border.itsGNResponses, false );
      showConnections( border.itsBOResponses[0], border.itsGNResponses, false );
      bd::showImage( visualizeInhibitoryPairs( border.itsBOResponses[0] ), false );
      displayPolarity(border.itsBOResponses[0], border.itsGNResponses, false) ;
      //doMask();
    } );
  }

  doMask();

  for( int y = 10; y < 20; ++y )
    presentSquare( 10, y, 13, [&]()
    {
      showConnections( border.itsGNResponses, false );
      showConnections( border.itsBOResponses[0], border.itsGNResponses, false );
      bd::showImage( visualizeInhibitoryPairs( border.itsBOResponses[0] ), false );
      displayPolarity(border.itsBOResponses[0], border.itsGNResponses, false) ;
      doMask();
    } );
}

void testSlide( bd::Border & border, cv::Mat const & mask )
{
  const int height = border.itsGridHeight;
  const int width = border.itsGridWidth;

  auto image = bd::generateRectangle( height, width, 10, 10, 10, 10 );

  auto doMask = [&]()
  {
    border.onset( mask );
    border.iterate( 20 );
  };

  auto presentSquare = [&]( int x, int y, int numIters, std::function<void()> postFunc )
  {
    auto localimage = bd::generateRectangle( height, width, x, y, 10, 10 );

    border.onset( localimage );
    border.iterate(numIters);

    postFunc();
  };

  for( int y = 10; y < height - 10; ++y )
  {
    presentSquare( 10, y, 9, [&]()
    {
      showConnections( border.itsGNResponses, false );
      //showConnections( border.itsGNIIResponses, false );
      showConnections( border.itsBOResponses[0], border.itsGNResponses, false );
      bd::showImage( visualizeInhibitoryPairs( border.itsBOResponses[0] ), false );
      displayPolarity(border.itsBOResponses[0], border.itsGNResponses, false) ;
      //doMask();
    } );
  }

  doMask();

  for( int y = 10; y < height - 10; ++y )
    presentSquare( 10, y, 9, [&]()
    {
      showConnections( border.itsGNResponses, false );
      showConnections( border.itsBOResponses[0], border.itsGNResponses, false );
      bd::showImage( visualizeInhibitoryPairs( border.itsBOResponses[0] ), false );
      displayPolarity(border.itsBOResponses[0], border.itsGNResponses, false) ;
      doMask();
    } );
}

void testSlideHorizontal( bd::Border & border, cv::Mat const & mask )
{
  const int numIters = 6;

  for( int i = 0; i < numIters; ++i )
  {
    for( int x = 10; x < 14; ++x )
    {
      auto localimage = bd::generateRectangle( 40, 40, x, 10, 10, 10 );

      border.onset( localimage );
      border.iterate(100);

      if( i == numIters - 1 )
      {
      showConnections( border.itsGNResponses, false );
      showConnections( border.itsBOResponses[0], border.itsGNResponses, false );
      bd::showImage( visualizeInhibitoryPairs( border.itsBOResponses[0] ), false );
      }
    }

    border.onset( mask );
    border.iterate(10);
  }

  std::cout << "*************" << std::endl;
  std::cout << "*************" << std::endl;
  std::cout << "*************" << std::endl;
  std::cout << "*************" << std::endl;
  std::cout << "*************" << std::endl;
  std::cout << "*************" << std::endl;
  std::cout << "*************" << std::endl;

  //border.setPeriod( bd::SystemState::Period::Transition );
  //std::cout << "---------------- TRANSITION -----------------" << std::endl;

  //for( int i = 0; i < numIters; ++i )
  //{
  //  for( int x = 10; x < 15; ++x )
  //  {
  //    auto localimage = bd::generateRectangle( 40, 40, x, 10, 10, 10 );

  //    border.onset( localimage );
  //    border.iterate(100);

  //    showConnections( border.itsGNResponses, true );
  //    showConnections( border.itsBOResponses[0], border.itsGNResponses, false );
  //    bd::showImage( visualizeInhibitoryPairs( border.itsBOResponses[0] ), false );

  //    border.onset( mask );
  //    border.iterate(10);
  //  }
  //}

  //border.setPeriod( bd::SystemState::Period::Stable );
  std::cout << "---------------- STABLE -----------------" << std::endl;

  for( int x = 10; x < 11; ++x )
  {
    //std::cout << "-----------------------------------------------" << std::endl;
    auto localimage = bd::generateRectangle( 40, 40, x, 10, 10, 10 );

    border.onset( localimage );
    border.iterate(10);

    showConnections( border.itsGNResponses, false );
    showConnections( border.itsBOResponses[0], border.itsGNResponses, false );
    bd::showImage( visualizeInhibitoryPairs( border.itsBOResponses[0] ), false );

    border.onset( mask );
    border.iterate(10);
  }
}

void testSlideSquare( bd::Border & border, cv::Mat const & mask )
{
  const int numIters = 6;

  auto presentSquare = [&]( int x, int y, std::function<void()> postFunc )
  {
    auto localimage = bd::generateRectangle( 40, 40, x, y, 10, 10 );

    border.onset( localimage );
    border.iterate(100);

    postFunc();
  };

  auto doSquare = [&](std::function<void()> postFunc)
  {
    for( int x = 10; x < 20; ++x )
      presentSquare( x, 10, postFunc );

    for( int y = 11; y < 20; ++y )
      presentSquare( 19, y, postFunc );

    for( int x = 19; x >= 10; --x )
      presentSquare( x, 19, postFunc );

    for( int y = 18; y > 10; --y )
      presentSquare( 10, y, postFunc );
  };

  for( int i = 0; i < numIters; ++i )
  {
    doSquare( [](){} );
    border.onset( mask );
    border.iterate(3);
  }

  for( int i = 0; i < numIters; ++i )
  {
    doSquare( [&](){ border.onset( mask ); border.iterate(3); } );
  }

  doSquare(
      [&]()
      {
        showConnections( border.itsGNResponses, false );
        showConnections( border.itsBOResponses[0], border.itsGNResponses, false );
        bd::showImage( visualizeInhibitoryPairs( border.itsBOResponses[0] ), false );

        border.onset( mask );
        border.iterate(3);
      } );
}

