#ifndef SIMPLETESTS_H_
#define SIMPLETESTS_H_

#include "../src/Border.H"

//! Present a single stimulus, viewing after each presentation
/*! This also outputs text files containing the current activation values
    for each border ownership neuron.
    @param numIters number of iterations to present a stimulus for
    @param skip number of iterations to present stimulus before first viewing */
void testConstant( bd::Border & border, cv::Mat const & stimulus,
    int numIters, int skip );

//! Presents stimulus followed by mask followed by stimulus again
void testOnOff( bd::Border & border, cv::Mat const & stimulus, cv::Mat const & mask );

//! Presents a square followed by its rival (share a common border)
void testFlip( bd::Border & bd, double scale = 1.0 );

//! Presents a square that slides into position of its rival
void testFlipSlide( bd::Border & bd, cv::Mat const & mask );

//! Continually slides down
void testSlide( bd::Border & bd, cv::Mat const & mask );

//! Presents a sliding square
/*! Square initially slides continuously with mask presented after a chunk of
    movement.

    Second phase presents mask in between each position.

    Third (viewing) phase presents full mask in between each position */
void testSlideHorizontal( bd::Border & border, cv::Mat const & mask );

//! Acts as the above horizontal except the square is moved around in a suare trajectory
void testSlideSquare( bd::Border & border, cv::Mat const & mask );

#endif // SIMPLETESTS_H_
