#ifndef V1TESTS_H_
#define V1TESTS_H_

#include "../src/V1.H"

/*!
    @param stdBias The standard deviation of the random line distribution (centered on vertical).
                   Pass a negative number to use a natural distribution.
    @param inputScaling Scaling to apply to the resolution of the input image
 */
void testV1Convergence( bd::V1 & v1, cv::Mat const & mask, double stdBias, int inputScaling, std::string const & fileSuffix );

//! Creates a colored image where each pixel represents the preferred orientation for a v1 edge neuron
/*! Brightness is based on how well learned the neuron is
    @param inputScaling Scaling to apply to the resolution of the input image */
template <class Network> inline
cv::Mat getV1OrientationPreferencesImage( Network & v1, int inputScaling );

//! Creates a colored image where each pixel represents the preferred orientation
/*! Uses MATLAB to do the fitting, brightness NOT based on how well learned it is.
    MATLAB directly fits a Gabor to the receptive field and does not present any input
    @return The image generated as well as its learning score */
template <class Network> inline
std::pair<cv::Mat, float> getV1OrientationPreferencesImageMatlab( Network & v1 );
#endif // V1TESTS_H_
